# $Id$
#
# Nose program for testing grizzled.config.Configuration

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

from grizzled.config import (Configuration, NoVariableError)
from cStringIO import StringIO
import os
import tempfile
import atexit
import sys

# ---------------------------------------------------------------------------
# Globals
# ---------------------------------------------------------------------------

CONFIG1 = """
[section1]
foo = bar
bar = ${foo}
bar2 = ${section1:foo}
name = ${program:name}
time = ${program:now}
cwd = ${program:cwd}
"""

CONFIG2 = """
[section1]
foo = bar
bar = ${foo}

[section2]
foo = ${section1:foo}
bar = ${env:SOME_ENV_VAR}
"""

CONFIG_ORDER_TEST = """
[z]
foo = bar
bar = ${foo}

[y]
foo = ${z:foo}
bar = ${z:bar}

[a]
foo = 1
bar = 2

[z2]
foo = ${z:foo}
bar = ${z:bar}
"""

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class TestParser(object):

    def testSubstitute1(self):
        config = Configuration()
        config.readfp(StringIO(CONFIG1))
        assert config.has_section('section1')
        assert not config.has_section('section2')
        assert not config.has_section('foo')
        assert not config.has_section('bar')
        assert not config.has_section('bar2')
        assert config.has_option('section1', 'foo')
        assert config.has_option('section1', 'name')
        assert config.get('section1', 'name') == os.path.basename(sys.argv[0])
        assert config.get('section1', 'cwd') == os.getcwd()
        assert config.has_option('section1', 'bar')
        assert config.has_option('section1', 'bar2')
        assert config.get('section1', 'foo') == 'bar'
        assert config.get('section1', 'bar') == 'bar'
        assert config.get('section1', 'bar2') == 'bar'

    def testSubstitute2(self):
        os.environ['SOME_ENV_VAR'] = 'test_test_test'
        config = Configuration()
        config.readfp(StringIO(CONFIG2))
        assert config.has_section('section1')
        assert config.has_section('section2')
        assert not config.has_section('foo')
        assert not config.has_section('bar')
        assert not config.has_section('bar2')
        assert config.has_option('section1', 'foo')
        assert config.has_option('section1', 'bar')
        assert not config.has_option('section1', 'bar2')
        assert config.has_option('section2', 'foo')
        assert config.has_option('section2', 'bar')
        assert config.get('section1', 'foo') == 'bar'
        assert config.get('section1', 'bar') == 'bar'
        assert config.get('section2', 'foo') == 'bar'
        assert config.get('section2', 'bar') == os.environ['SOME_ENV_VAR']

    def testInclude(self):
        fd, tempPath = tempfile.mkstemp(suffix='.cfg')

        def unlinkTemp(path):
            try:
                os.unlink(path)
            except:
                pass

        atexit.register(unlinkTemp, tempPath)
        fp = os.fdopen(fd, "w")
        print >> fp, '[section3]\nbaz = somevalue\n'
        fp.close()

        s = '%s\n\n%%include "%s"\n' % (CONFIG2, tempPath)

        os.environ['SOME_ENV_VAR'] = 'test_test_test'
        config = Configuration()
        config.readfp(StringIO(s))
        unlinkTemp(tempPath)
        assert config.has_section('section1')
        assert config.has_section('section2')
        assert config.has_section('section3')
        assert not config.has_section('foo')
        assert not config.has_section('bar')
        assert not config.has_section('bar2')
        assert config.has_option('section1', 'foo')
        assert config.has_option('section1', 'bar')
        assert not config.has_option('section1', 'bar2')
        assert config.has_option('section2', 'foo')
        assert config.has_option('section2', 'bar')
        assert config.has_option('section3', 'baz')
        assert config.get('section1', 'foo') == 'bar'
        assert config.get('section1', 'bar') == 'bar'
        assert config.get('section2', 'foo') == 'bar'
        assert config.get('section2', 'bar') == os.environ['SOME_ENV_VAR']
        assert config.get('section3', 'baz') == 'somevalue'

    def testOrdering(self):
        config = Configuration(use_ordered_sections=True)
        config.readfp(StringIO(CONFIG_ORDER_TEST))
        assert config.has_section('a')
        assert config.has_section('y')
        assert config.has_section('z')
        sections = config.sections
        assert len(sections) == 4
        assert sections[0] == 'z'
        assert sections[1] == 'y'
        assert sections[2] == 'a'
        assert sections[3] == 'z2'

    def testBadSubstitution(self):
        cfgString = """
[foo]
var1 = ${bar}
"""
        import sys
        from grizzled.io import AutoFlush
        sys.stdout = AutoFlush(sys.stdout)
        config = Configuration(strict_substitution=False)
        config.readfp(StringIO(cfgString))
        config.write(sys.stdout)

        try:
            var1 = config.get('foo', 'var1', optional=True)
            assert var1 == None, 'Expected empty variable value'
        except:
            raise

        config = Configuration(strict_substitution=True)
        try:
            config.readfp(StringIO(cfgString))
            assert False, 'Expected an exception'
        except NoVariableError:
            pass
        except:
            assert False, 'Unexpected exception'
        



#!/usr/bin/env python

# NOTE: Documentation is intended to be processed by epydoc and contains
# epydoc markup.

'''
Introduction
============

The ``grizzled.file.includer`` module contains a class that can be used to
process includes within a text file, returning a file-like object. It also
contains some utility functions that permit using include-enabled files in
other contexts.

Include Syntax
==============

The *include* syntax is defined by a regular expression; any line that matches
the regular expression is treated as an *include* directive. The default
regular expression matches include directives like this::

    %include "/absolute/path/to/file"
    %include "../relative/path/to/file"
    %include "local_reference"
    %include "http://localhost/path/to/my.config"

Relative and local file references are relative to the including file or URL.
That, if an ``Includer`` is processing file "/home/bmc/foo.txt" and encounters
an attempt to include file "bar.txt", it will assume "bar.txt" is to be found
in "/home/bmc".

Similarly, if an ``Includer`` is processing URL "http://localhost/bmc/foo.txt"
and encounters an attempt to include file "bar.txt", it will assume "bar.txt"
is to be found at "http://localhost/bmc/bar.txt".

Nested includes are permitted; that is, an included file may, itself, include
other files. The maximum recursion level is configurable and defaults to 100.

The include syntax can be changed by passing a different regular expression to
the ``Includer`` class constructor.

Usage
=====

This module provides an ``Includer`` class, which processes include directives
in a file and behaves like a file-like object. See the class documentation for
more details.

The module also provides a ``preprocess()`` convenience function that can be
used to preprocess a file; it returns the path to the resulting preprocessed
file.

Examples
========

Preprocess a file containing include directives, then read the result:

.. python::

    import includer
    import sys

    inc = includer.Includer(path)
    for line in inc:
        sys.stdout.write(line)


Use an include-enabled file with the standard Python logging module:

.. python::

    import logging
    import includer

    logging.fileConfig(includer.preprocess("mylog.cfg"))

'''

__docformat__ = "restructuredtext en"
__all__ = ['Includer', 'IncludeError', 'preprocess']

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import logging
import os
import sys
import re
import tempfile
import atexit
import urllib2
import urlparse

import grizzled.exception
from grizzled.file import unlink_quietly

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['IncludeError', 'Includer', 'preprocess']

# ---------------------------------------------------------------------------
# Logging
# ---------------------------------------------------------------------------

log = logging.getLogger('includer')

# ---------------------------------------------------------------------------
# Public classes
# ---------------------------------------------------------------------------

class IncludeError(grizzled.exception.ExceptionWithMessage):
    """
    Thrown by ``Includer`` when an error occurs while processing the file.
    An ``IncludeError`` object always contains a single string value that
    contains an error message describing the problem.
    """
    pass

class Includer(object):
    '''
    An ``Includer`` object preprocesses a path or file-like object,
    expanding include references. The resulting ``Includer`` object is a
    file-like object, offering the same methods and capabilities as an open
    file.

    By default, ``Includer`` supports this include syntax::

        %include "path"
        %include "url"

    However, the include directive syntax is controlled by a regular
    expression, so it can be configured.

    See the module documentation for details.
    '''
    def __init__(self,
                 source,
                 include_regex='^%include\s"([^"]+)"',
                 max_nest_level=100,
                 output=None):
        """
        Create a new ``Includer`` object.

        :Parameters:
            source : file or str
                The source to be read and expanded. May be an open file-like
                object, a path name, or a URL string.
            include_regex : str
                Regular expression defining the include syntax. Must contain a
                single parenthetical group that can be used to extract the
                included file or URL.
            max_nest_level : int
                Maximum include nesting level. Exceeding this level will cause
                ``Includer`` to throw an ``IncludeError``.
            output : str or file
                A string (path name) or file-like object to which to save the
                expanded output.

        :raise IncludeError: On error
        """

        if isinstance(source, str):
            f, is_url, name = self.__open(source, None, False)
        else:
            # Assume file-like object.
            f = source
            is_url = False
            try:
                name = source.name
            except AttributeError:
                name = None

        self.closed = False
        self.mode = None
        self.__include_pattern = re.compile(include_regex)
        self.__name = name

        if output == None:
            from cStringIO import StringIO
            output = StringIO()

        self.__maxnest = max_nest_level
        self.__nested = 0
        self.__process_includes(f, name, is_url, output)
        self.__f = output
        self.__f.seek(0)

    @property
    def name(self):
        """
        Get the name of the file being processed.
        """
        return self.__name

    def __iter__(self):
        return self

    def next(self):
        """A file object is its own iterator.

        :rtype: string
        :return: the next line from the file

        :raise StopIteration: end of file
        :raise IncludeError: on error
        """
        line = self.readline()
        if (line == None) or (len(line) == 0):
            raise StopIteration
        return line

    def close(self):
        """Close the includer, preventing any further I/O operations."""
        if not self.closed:
            self.closed = true
            self.__f.close()
            del self.__f

    def fileno(self):
        """
        Get the file descriptor. Returns the descriptor of the file being
        read.

        :rtype:  int
        :return: the file descriptor of the file being read
        """
        _complain_if_closed(self.closed)
        return self.__f.fileno()

    def isatty(self):
        """
        Determine whether the file being processed is a TTY or not.

        :return: ``True`` or ``False``
        """
        _complain_if_closed(self.closed)
        return self.__f.isatty()

    def seek(self, pos, mode=0):
        """
        Seek to the specified file offset in the include-processed file.

        :Parameters:
            pos : int
                file offset
            mode : int
                the seek mode, as specified to a Python file's ``seek()``
                method
        """
        self.__f.seek(pos, mode)

    def tell(self):
        """
        Get the current file offset.

        :rtype:  int
        :return: current file offset
        """
        _complain_if_closed(self.closed)
        return self.__f.tell()

    def read(self, n=-1):
        """
        Read *n* bytes from the open file.

        :Parameters:
            n : int
                Number of bytes to read. A negative number instructs
                the method to read all remaining bytes.

        :return: the bytes read
        """
        _complain_if_closed(self.closed)
        return self.__f.read(n)

    def readline(self, length=-1):
        """
        Read the next line from the file.

        :Parameters:
            length : int
                a length hint, or negative if you don't care
                
        :rtype: str
        :return: the line read
        """
        _complain_if_closed(self.closed)
        return self.__f.readline(length)

    def readlines(self, sizehint=0):
        """
        Read all remaining lines in the file.

        :rtype:  array
        :return: array of lines
        """
        _complain_if_closed(self.closed)
        return self.__f.readlines(sizehint)

    def truncate(self, size=None):
        """Not supported, since ``Includer`` objects are read-only."""
        raise IncludeError, 'Includers are read-only file objects.'

    def write(self, s):
        """Not supported, since ``Includer`` objects are read-only."""
        raise IncludeError, 'Includers are read-only file objects.'

    def writelines(self, iterable):
        """Not supported, since ``Includer`` objects are read-only."""
        raise IncludeError, 'Includers are read-only file objects.'

    def flush(self):
        """No-op."""
        pass

    def getvalue(self):
        """
        Retrieve the entire contents of the file, which includes expanded,
        at any time before the ``close()`` method is called.

        :rtype:  string
        :return: a single string containing the contents of the file
        """
        return ''.join(self.readlines())

    def __process_includes(self, file_in, filename, is_url, file_out):
        log.debug('Processing includes in "%s", is_url=%s' % (filename, is_url))

        for line in file_in:
            match = self.__include_pattern.search(line)
            if match:
                if self.__nested >= self.__maxnest:
                    raise IncludeError, 'Exceeded maximum include recursion ' \
                                        'depth of %d' % self.__maxnest

                inc_name = match.group(1)
                logging.debug('Found include directive: %s' % line[:-1])
                f, included_is_url, included_name = self.__open(inc_name,
                                                                filename,
                                                                is_url)
                self.__nested += 1
                self.__process_includes(f, filename, is_url, file_out)
                self.__nested -= 1
            else:
                file_out.write(line)

    def __open(self, name_to_open, enclosing_file, enclosing_file_is_url):
        is_url = False
        openFunc = None

        parsed_url = urlparse.urlparse(name_to_open)

        # Account for Windows drive letters.

        if (parsed_url.scheme != '') and (len(parsed_url.scheme) > 1):
            openFunc = urllib2.urlopen
            is_url = True

        else:
            # It's not a URL. What we do now depends on the including file.

            if enclosing_file_is_url:
                # Use the parent URL as the base URL.
                
                name_to_open = urlparse.urljoin(enclosing_file, name_to_open)
                open_func = urllib2.urlopen
                is_url = True

            elif not os.path.isabs(name_to_open):
                # Not an absolute file. Base it on the parent.

                enclosing_dir = None
                if enclosing_file == None:
                    enclosing_dir = os.getcwd()
                else:
                    enclosing_dir = os.path.dirname(enclosing_file)

                name_to_open = os.path.join(enclosing_dir, name_to_open)
                open_func = open

            else:
                open_func = open

        assert(name_to_open != None)
        assert(open_func != None)

        try:
            log.debug('Opening "%s"' % name_to_open)
            f = open_func(name_to_open)
        except:
            raise IncludeError, 'Unable to open "%s" as a file or a URL' %\
                  name_to_open
        return (f, is_url, name_to_open)
    
# ---------------------------------------------------------------------------
# Public functions
# ---------------------------------------------------------------------------

def preprocess(file_or_url, output=None, temp_suffix='.txt', temp_prefix='inc'):
    """
    Process all include directives in the specified file, returning a path
    to a temporary file that contains the results of the expansion. The
    temporary file is automatically removed when the program exits, though
    the caller is free to remove it whenever it is no longer needed.

    :Parameters:
        file_or_url : file or str
            URL or path to file to be expanded; or, a file-like object
        output : file
            A file or file-like object to receive the output.
        temp_suffix : str
            suffix to use with temporary file that holds preprocessed output
        temp_prefix : str
            prefix to use with temporary file that holds preprocessed output

    :rtype:  string
    :return: ``output``, if ``output`` is not ``None``; otherwise, the path to
             temporary file containing expanded content
    """
    result = None
    path = None
    if not output:
        fd, path = tempfile.mkstemp(suffix=temp_suffix, prefix=temp_prefix)
        output = open(path, 'w')
        atexit.register(unlink_quietly, path)
        os.close(fd)
        result = path
    else:
        result = output

    Includer(file_or_url, output=output)
    return result

    
# ---------------------------------------------------------------------------
# Private functions
# ---------------------------------------------------------------------------

def _complain_if_closed(closed):
    if closed:
        raise IncludeError, "I/O operation on closed file"

# ---------------------------------------------------------------------------
# Main program (for testing)
# ---------------------------------------------------------------------------

if __name__ == '__main__':

    format = '%(asctime)s %(name)s %(levelname)s %(message)s'
    logging.basicConfig(level=logging.DEBUG, format=format)

    for file in sys.argv[1:]:
        import cStringIO as StringIO
        out = StringIO.StringIO()
        preprocess(file, output=out)
        
        header = 'File: %s, via preprocess()'
        sep = '-' * len(header)
        print '\n%s\n%s\n%s\n' % (sep, header, sep)
        for line in out.readlines():
            sys.stdout.write(line)
        print sep

        inc = Includer(file)
        header = 'File: %s, via Includer'
        sep = '-' * len(header)
        print '\n%s\n%s\n%s\n' % (sep, header, sep)
        for line in inc:
            sys.stdout.write(line)
        print '%s' % sep



# $Id$

"""
Provides some base exception classes.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['ExceptionWithMessage']

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class ExceptionWithMessage(Exception):
    """
    Useful base class for exceptions that have a single exception message
    argument. Among other things, this method provides a reasonable default
    ``__str__()`` method.

    Usage:
    
    .. python::

        from grizzled.exception import ExceptionWithMessage

        class MyException(ExceptionWithMessage):
            def __init__(self, msg):
                ExceptionWithMessage.__init__(self, msg)
    """
    def __init__(self, errorMessage):
        """
        Create a new exception.

        @type errorMessage:  string
        @param errorMessage: the error message
        """
        self.__message = errorMessage

    @property
    def message(self):
        """
        The message stored with this object.
        """
        return self.__message

    def __str__(self):
        return '%s: %s' % (self.__class__.__name__, self.__message)



"""
``grizzled.history`` provides a command line history capability that
provides the same interface across different history implementations.
Currently, it supports three history implementations:

- `GNU Readline`_, which is built into versions of Python on the Mac
  and Unix systems
- `pyreadline`_, which many people use on Windows systems
- A dummy fallback history implementation that does nothing, for when readline
  isn't available.
  
The `History` class provides the interface and some common methods for
all history operations.

.. _pyreadline: http://ipython.scipy.org/dist/
.. _GNU Readline: http://cnswww.cns.cwru.edu/php/chet/readline/rluserman.html

To get the appropriate History implementation for the current platform,
simply call the ``get_history()`` factory method.
"""

from __future__ import with_statement

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import re
import sys
import logging
import copy

from grizzled.decorators import abstract
from grizzled.exception import ExceptionWithMessage

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['get_history', 'History', 'DEFAULT_MAXLENGTH', 'HistoryError']
__docformat__ = 'restructuredtext'

# ---------------------------------------------------------------------------
# Constants
# ---------------------------------------------------------------------------

DEFAULT_MAXLENGTH = 512

# ---------------------------------------------------------------------------
# Globals
# ---------------------------------------------------------------------------

log = logging.getLogger('history')
_have_readline = False
_have_pyreadline = False

try:
    import readline
    _have_readline = True

    # Is it pyreadline? If so, it's not quite the same.

    try:
        _have_pyreadline = readline.rl.__module__.startswith('pyreadline.')
    except AttributeError:
        pass
except ImportError:
    pass

# ---------------------------------------------------------------------------
# Functions
# ---------------------------------------------------------------------------

def get_history(verbose=True):
    """
    Factory method to create an appropriate History object.

    :Parameters:
        verbose : bool
            ``True`` to display a message on standard output about what
            history management mechanism is being used.

    :rtype: ``History``
    :return: the ``History`` object
    """
    global _have_readline
    global _have_pyreadline
    result = None
    if _have_pyreadline:
        if verbose:
            print 'Using pyreadline for history management.'
        result = PyReadlineHistory()

    elif _have_readline:
        if verbose:
            print 'Using readline for history management.'
        result = ReadlineHistory()

    else:
        print 'WARNING: Readline unavailable. There will be no history.'
        result = DummyHistory()

    result.max_length = DEFAULT_MAXLENGTH
    return result

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class HistoryError(ExceptionWithMessage):
    """
    Thrown to indicate history errors, when another exception won't do.
    """
    pass

class History(object):
    """
    Base class for history implementations.  All concrete history
    implementations must extend this class.
    """
    def __init__(self):
        self.set_max_length(DEFAULT_MAXLENGTH)

    def show(self, out=sys.stdout):
        """
        Dump the history to a file-like object (defaulting to standard output).

        :Parameters:
            out : file
                Where to dump the history.
        """
        for i in range(1, self.total + 1):
            print >> out, '%4d: %s' % (i, self.get_item(i))

    def get_last_matching_item(self, command_name):
        """
        Get the most recently entered item that matches ``command_name``
        at the beginning.

        :Parameters:
            command_name : str
                The string to match against the commands in the history

        :rtype: str
        :return: the matching string, or ``None``
        """
        result = None
        for i in range(self.get_total(), 0, -1):
            s = self.get_item(i)
            tokens = s.split(None, 1)
            if len(command_name) <= len(s):
                if s[0:len(command_name)] == command_name:
                    result = s
                    break
        return result

    def get_last_item(self):
        """
        Get the most recent item in the history.

        :rtype: str
        :return: The most recent command, or ``None``
        """
        return self.get_item(self.get_total() - 1)

    def get_item(self, index):
        """
        Get an item from the history.

        :Parameters:
            index : int
                0-based index of the item to get. The larger the index
                value, the more recent the entry

        :rtype: str
        :return: the item at that index

        :raise IndexError: Index out of range
        """
        return None

    def set_completer_delims(self, s):
        """
        Set the completer delimiters--the characters that delimit tokens
        that are eligible for completion.

        :Parameters:
            s : str
                The delimiters
        """
        pass

    def get_completer_delims(self):
        """
        Get the completer delimiters--the characters that delimit tokens
        that are eligible for completion.

        :rtype: str
        :return: the delimiters
        """
        return ''

    @property
    def total(self):
        """
        The total number number of commands in the history. Identical to
        calling ``get_total()``.
        """
        return self.get_total()

    def get_total(self):
        """
        Get the total number number of commands in the history. Identical to
        the ``total`` property.

        :rtype: int
        :return: the number of commands in the history
        """
        return 0

    def __set_max_length(self, n):
        return self.set_max_length(n)

    def __get_max_length(self):
        return self.get_max_length()

    maxLength = property(__get_max_length, __set_max_length,
                         doc="The maximum length of the history")

    @abstract
    def get_max_length(self):
        """
        Get the maximum length of the history. This isn't the maximum number
        of entries in the in-memory history buffer; instead, it's the maximum
        number of entries that will be saved to the history file. Subclasses
        *must* provide an implementation of this method.

        :rtype: int
        :return: the maximum saved size of the history
        """
        pass

    @abstract
    def set_max_length(self, n):
        """
        Set the maximum length of the history. This isn't the maximum number
        of entries in the in-memory history buffer; instead, it's the maximum
        number of entries that will be saved to the history file.  Subclasses
        *must* provide an implementation of this method.

        :Parameters:
            n : int
                the maximum saved size of the history
        """
        pass

    @abstract
    def add_item(self, line):
        """
        Add (append) a line to the history buffer. Subclasses *must* provide
        an implementation of this method.

        :Parameters:
            line : str
                the command to append to the history
        """
        pass

    @abstract
    def remove_item(self, i):
        """
        Remove a line from the history buffer. Subclasses *must* provide an
        implementation of this method.

        :Parameters:
            i : int
                the 0-based index of the item to be removed
        """
        pass

    @abstract
    def clear_history(self):
        """
        Clear the history buffer. Subclasses *must* provide an
        implementation of this method.
        """
        pass

    def get_history_list(self):
        """
        Get a copy of the history buffer.

        :rtype: list
        :return: a list of commands from the history
        """
        result = []
        for i in range(1, self.total + 1):
            result += [self.get_item(i)]

        return result

    def remove_matches(self, regexp_string):
        """
        Remove all history items that match a regular expression.

        :Parameters:
            regexp_string : str
                the uncompiled regular expression to match

        :raise HistoryError: bad regular expression
        """
        try:
            pat = re.compile(regexp_string)
        except:
            raise HistoryError(str(sys.exc_info[1]))

        buf = []

        for i in range(1, self.total + 1):
            s = self.get_item(i)
            if not pat.match(s):
                buf += [s]

        self.replace_history(buf)

    def cut_back_to(self, index):
        """
        Cut the history back to the specified index, removing all entries
        more recent than that index.

        :Parameters:
            index : int
                the index of the command that should become the last command
                in the history

        :raise IndexError: index out of range
        """
        if (index > 0) and (index <= self.total):
            buf = []
            for i in range(1, index):
                buf += [self.get_item(i)]

            self.replace_history(buf)

    def replace_history(self, commands):
        """
        Replace the entire contents of the history with another set of values

        :Parameters:
            commands : list
                List of strings to put in the history after clearing it of any
                existing entries
        """
        self.clear_history()
        for command in commands:
            self.add_item(command, force=True)

    def save_history_file(self, path):
        """
        Save the history to a file. The file is overwritten with the contents
        of the history buffer.

        :Parameters:
            path : str
                Path to the history file to receive the output.

        :raise IOError: Unable to open file
        """
        log.debug('Writing history file "%s"' % path)
        with open(path, 'w') as f:
            for i in range(1, self.total + 1):
                f.write(self.get_item(i) + '\n')

    def load_history_file(self, path):
        """
        Load the history buffer with the contents of a file, completely
        replacing the in-memory history with the file's contents.

        :Parameters:
            path : str
                Path to the history file to read

        :raise IOError: Unable to open file
        """
        log.debug('Loading history file "%s"' % path)
        with open(path, 'r') as f:
            buf = []
            for line in f:
                buf += [line.strip()]

        max = self.get_max_length()
        if len(buf) > max:
            buf = buf[max]
        self.replace_history(buf)

class ReadlineHistory(History):

    def __init__(self):
        global _have_readline
        assert(_have_readline)
        History.__init__(self)

    def get_item(self, index):
        return readline.get_history_item(index)

    def get_total(self):
        return readline.get_current_history_length()

    def set_completer_delims(self, s):
        readline.set_completer_delims(s)

    def get_completer_delims(self,):
        return readline.get_completer_delims()

    def remove_item(self, index):
        # readline.remove_history_item() doesn't seem to work. Do it the
        # hard way.

        #try:
        #    readline.remove_history_item(i)
        #except ValueError:
        #    pass

        buf = []
        for i in range(1, self.total + 1):
            if i != index:
                buf += self.get_item(i)

        self.clear_history()
        for s in buf:
            readline.add_history(s)

    def clear_history(self):
        try:
            readline.clear_history()
        except AttributeError:
            len = self.get_max_length()
            readline.set_history_length(0)
            readline.set_history_length(len)

    def get_max_length(self):
        return readline.get_history_length()

    def set_max_length(self, n):
        readline.set_history_length(n)

    def add_item(self, line, force=False):
        readline.add_history(line)

class PyReadlineHistory(ReadlineHistory):
    def __init__(self):
        global _have_pyreadline
        assert(_have_pyreadline)
        ReadlineHistory.__init__(self)

    def get_item(self, index):
        return self.__get_buf()[index - 1].get_line_text()

    def get_total(self):
        return len(self.__get_buf())

    def set_completer_delims(self, s):
        readline.set_completer_delims(s)

    def get_completer_delims(self):
        return readline.get_completer_delims()

    def remove_item(self, index):
        buf = copy.deepcopy(self.__get_buf())
        self.clear_history()
        for s in buf:
            readline.add_history(s)

    def clear_history(self):
        readline.clear_history()

    def get_max_length(self):
        return readline.get_history_length()

    def set_max_length(self, n):
        readline.set_history_length(n)

    def add_item(self, line, force=False):
        # Kludge. pyreadline is a pain in the ass.
        from pyreadline import lineobj
        from pyreadline.unicode_helper import ensure_unicode

        line = ensure_unicode(line.rstrip())
        readline.add_history(lineobj.ReadLineTextBuffer(line))

    def __get_buf(self):
        return readline.rl._history.history

class DummyHistory(History):

    def __init__(self):
        History.__init__(self)

    def remove_item(self, i):
        pass

    def get_item(self, index):
        return None

    def get_history_list(self):
        return []

    def get_total(self):
        return 0

    def get_max_length(self):
        return 0

    def set_max_length(self, n):
        pass

    def clear_history(self):
        pass

    def add_item(self, line, force=False):
        pass

# ---------------------------------------------------------------------------
# Main
# ---------------------------------------------------------------------------

if __name__ == "__main__":
    h = getHistory()



# $Id: 29411c26e95478f9d37f888b7720a8c37871c9a1 $

"""
Introduction
============

The ``db`` module is a DB API wrapper. It provides a DB API-compliant API that
wraps real underlying DB API drivers, simplifying some non-portable operations
like ``connect()`` and providing some new operations.

Some drivers come bundled with this package. Others can be added on the fly.

Getting the List of Drivers
===========================

To get a list of all drivers currently registered with this module, use the
``get_driver_names()`` method:

.. python::

    import db

    for driver_name in db.get_driver_names():
        print driver_name

Currently, this module provides the following bundled drivers:

  +------------------+------------+-------------------+
  | Driver Name,     |            |                   |
  | as passed to     |            | Underlying Python |
  | ``get_driver()`` | Database   | DB API module     |
  +==================+============+===================+
  | dummy            | None       | ``db.DummyDB``    |
  +------------------+------------+-------------------+
  | gadfly           | Gadfly     | ``gadfly``        |
  +------------------+------------+-------------------+
  | mysql            | MySQL      | ``MySQLdb``       |
  +------------------+------------+-------------------+
  | oracle           | Oracle     | ``cx_Oracle``     |
  +------------------+------------+-------------------+
  | postgresql       | PostgreSQL | ``psycopg2``      |
  +------------------+------------+-------------------+
  | sqlserver        | SQL Server | ``pymssql``       |
  +------------------+------------+-------------------+
  | sqlite           | SQLite 3   | ``sqlite3``       |
  +------------------+------------+-------------------+

To use a given driver, you must have the corresponding Python DB API module
installed on your system.

Adding a Driver
===============

It's possible to add a new driver to the list of drivers supplied by this
module. To do so:

 1. The driver class must extend ``DBDriver`` and provide the appropriate
    methods. See examples in this module.
 2. The driver's module (or the calling program) must register the driver
    with this module by calling the ``add_driver()`` function.


DB API Factory Functions
========================

The ``Binary()``, ``Date()``, ``DateFromTicks()``, ``Time()``,
``TimeFromTicks()``, ``TimeStamp()`` and ``TimestampFromTicks()`` DB API
functions can be found in the DB class. Thus, to make a string into a BLOB
with this API, you use:

.. python::

    driver = db.get_driver(driver_name)
    db = driver.connect(...)
    blob = db.Binary(some_string)
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import re
import time
import os
import sys
from datetime import date, datetime

from grizzled.exception import ExceptionWithMessage
from grizzled.decorators import abstract
from grizzled.db import (base, dummydb, dbgadfly, mysql, oracle, postgresql,
                         sqlite, sqlserver)
from grizzled.db.base import *

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['get_driver', 'add_driver', 'get_driver_names', 'DBDriver',
           'DB', 'Cursor', 'DBError', 'Error', 'Warning', 'apilevel',
           'threadsafety', 'paramstyle']

# ---------------------------------------------------------------------------
# Globals
# ---------------------------------------------------------------------------

DummyDriver = dummydb.DummyDriver
GadflyDriver = dbgadfly.GadflyDriver
MySQLDriver = mysql.MySQLDriver
OracleDriver = oracle.OracleDriver
PostgreSQLDriver = postgresql.PostgreSQLDriver
SQLite3Driver = sqlite.SQLite3Driver
SQLServerDriver = sqlserver.SQLServerDriver

drivers = { 'dummy'      : 'DummyDriver',
            'mysql'      : 'MySQLDriver',
            'postgresql' : 'PostgreSQLDriver',
            'sqlserver'  : 'SQLServerDriver',
            'sqlite'     : 'SQLite3Driver',
            'oracle'     : 'OracleDriver',
            'gadfly'     : 'GadflyDriver'}

apilevel = '2.0'
threadsafety = '1'
paramstyle = None

# ---------------------------------------------------------------------------
# Functions
# ---------------------------------------------------------------------------

def add_driver(key, driver_class, force=False):
    """
    Add a driver class to the list of drivers.

    :Parameters:
        key : str
            the key, also used as the driver's name
        driver_class : class
            the ``DBDriver`` subclass object
        force : bool
            ``True`` to force registration of the driver, even if there's an
            existing driver with the same key; ``False`` to throw an exception
            if there's an existing driver with the same key.

    :raise ValueError: There's an existing driver with the same key, and
                       ``force`` is ``False``
    """
    try:
        drivers[key]
        if not force:
            raise ValueError, 'A DB driver named "%s" is already installed' %\
                  key
    except KeyError:
        pass

    drivers[key] = driver_class

def get_drivers():
    """
    Get the list of drivers currently registered with this API. The result is
    a list of ``DBDriver`` subclasses. Note that these are classes, not
    instances. Once way to use the resulting list is as follows:

    .. python::

        for driver in db.get_drivers():
            print driver.__doc__

    :rtype:  list
    :return: list of ``DBDriver`` class names
    """
    return [str(d) for d in drivers.values()]

def get_driver_names():
    """
    Get the list of driver names currently registered with this API.
    Each of the returned names may be used as the first parameter to
    the ``get_driver()`` function.
    """
    return drivers.keys()

def get_driver(driver_name):
    """
    Get the DB API object for the specific database type. The list of
    legal database types are available by calling ``get_driver_names()``.

    :Parameters:
        driver_name : str
            name (key) of the driver

    :rtype: DBDriver
    :return: the instantiated driver

    :raise ValueError: Unknown driver name
    """
    try:
        o = drivers[driver_name]
        if type(o) == str:
            exec 'd = %s()' % o
        else:
            d = o()
        return d
    except KeyError:
        raise ValueError, 'Unknown driver name: "%s"' % driver_name




"""
``grizzled.collections.dict`` contains some useful dictionary classes
that extend the behavior of the built-in Python ``dict`` type.
"""
__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import sys

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['OrderedDict', 'LRUDict']

# ---------------------------------------------------------------------------
# Public Classes
# ---------------------------------------------------------------------------

                         ##############################                         
                         # OrderedDict implementation #                         
                         ##############################                         

class OrderedDict(dict):
    """
    ``OrderedDict`` is a simple ordered dictionary. The ``keys()``,
    ``items()``, ``__iter__()``, and other methods all return the keys in the
    order they were added to the dictionary. Note that re-adding a key (i.e.,
    replacing a key with a new value) does not change the original order.

    An ``OrderedDict`` object is instantiated with exactly the same parameters
    as a ``dict`` object. The methods it provides are identical to those in
    the ``dict`` type and are not documented here.
    """
    def __init__(self, *args, **kw):
        dict.__init__(self, *args, **kw)
        self.__orderedKeys = []
        self.__keyPositions = {}

    def __setitem__(self, key, value):
        try:
            index = self.__keyPositions[key]
        except KeyError:
            index = len(self.__orderedKeys)
            self.__orderedKeys += [key]
            self.__keyPositions[key] = index

        dict.__setitem__(self, key, value)

    def __delitem__(self, key):
        index = self.__keyPositions[key]
        del self.__orderedKeys[index]
        del self.__keyPositions[key]
        dict.__delitem__(self, key)

    def __iter__(self):
        for key in self.__orderedKeys:
            yield key

    def __str__(self):
        s = '{'
        sep = ''
        for k, v in self.iteritems():
            s += sep
            if type(k) == str:
                s += "'%s'" % k
            else:
                s += str(k)

            s += ': '
            if type(v) == str:
                s += "'%s'" % v
            else:
                s += str(v)
            sep = ', '
        s += '}'
        return s

    def keys(self):
        return self.__orderedKeys

    def items(self):
        return [(key, self[key]) for key in self.__orderedKeys]

    def values(self):
        return [self[key] for key in self.__orderedKeys]

    def iteritems(self):
        for key in self.__orderedKeys:
            yield (key, self[key])

    def iterkeys(self):
        for key in self.__orderedKeys:
            yield key

    def update(self, d):
        for key, value in d.iteritems():
            self[key] = value

    def pop(self, key, default=None):
        try:
            result = self[key]
            del self[key]

        except KeyError:
            if not default:
                raise

            result = default

        return result

    def popitem(self):
        key, value = dict.popitem(self)
        del self[key]
        return (key, value)

                           ##########################                           
                           # LRUDict Implementation #                           
                           ##########################                           

# Implementation note:
#
# Each entry in the LRUDict dictionary is an LRUListEntry. Basically,
# we maintain two structures:
#
# 1. A linked list of dictionary entries, in order of recency.
# 2. A dictionary of those linked list items.
#
# When accessing or updating an entry in the dictionary, the logic
# is more or less like the following "get" scenario:
#
# - Using the key, get the LRUListEntry from the dictionary.
# - Extract the value from the LRUListEntry, to return to the caller.
# - Move the LRUListEntry to the front of the recency queue.

class LRUListEntry(object):

    def __init__(self, key, value):
        self.key = key
        self.value = value
        self.next = None
        self.prev = None

    def __hash__(self):
        return self.key.__hash__()
    
    def __str__(self):
        return '(%s, %s)' % (self.key, self.value)

    def __repr__(self):
        return str(self)

class LRUList(object):

    def __init__(self):
        self.head = self.tail = None
        self.size = 0

    def __del__(self):
        self.clear()

    def __str__(self):
        return '[' + ', '.join([str(tup) for tup in self.items()]) + ']'
    
    def __repr__(self):
        return self.__class__.__name__ + ':' + str(self)

    def __len__(self):
        return self.size

    def __iter__(self):
        entry = self.head
        while entry:
            yield entry.key
            entry = entry.next

    def keys(self):
        return [k for k in self]

    def items(self):
        result = []
        for key, value in self.iteritems():
            result.append((key, value))
        return result

    def values(self):
        result = []
        for key, value in self.iteritems():
            result.append(value)
        return result

    def iteritems(self):
        entry = self.head
        while entry:
            yield (entry.key, entry.value)
            entry = entry.next

    def iterkeys(self):
        self.__iter__()

    def itervalues(self):
        entry = self.head
        while entry:
            yield entry.value
            entry = entry.next

    def clear(self):
        while self.head:
            cur = self.head
            next = self.head.next
            cur.next = cur.previous = cur.key = cur.value = None
            self.head = next

        self.tail = None
        self.size = 0

    def remove(self, entry):
        if entry.next:
            entry.next.previous = entry.previous

        if entry.previous:
            entry.previous.next = entry.next

        if entry == self.head:
            self.head = entry.next

        if entry == self.tail:
            self.tail = entry.previous

        entry.next = entry.previous = None
        self.size -= 1
        assert self.size >= 0

    def remove_tail(self):
        result = self.tail

        if result:
            self.remove(result)

        return result

    def add_to_head(self, entry):
        if type(entry) == tuple:
            key, value = entry
            entry = LRUListEntry(key, value)
        else:
            entry.next = entry.previous = None

        if self.head:
            assert self.tail
            entry.next = self.head
            self.head.previous = entry
            self.head = entry

        else:
            assert not self.tail
            self.head = self.tail = entry

        self.size += 1

    def move_to_head(self, entry):
        self.remove(entry)
        self.add_to_head(entry)

class LRUDict(dict):
    """
    ``LRUDict`` is a dictionary of a fixed maximum size that enforces a least
    recently used discard policy. When the dictionary is full (i.e., contains
    the maximum number of entries), any attempt to insert a new entry causes
    one of the least recently used entries to be discarded.

    **Note**:

    - Setting or updating a key in the dictionary refreshes the corresponding
      value, making it "new" again, even if it replaces the existing value with
      itself.
    - Retrieving a value from the dictionary also refreshes the entry.
    - Iterating over the contents of the dictionary (via ``in`` or ``items()``
      or any other similar method) does *not* affect the recency of the
      dictionary's contents.
    - This implementation is *not* thread-safe.

    An ``LRUDict`` also supports the concept of *removal listeners*. Removal
    listeners are functions that are notified when objects are removed from
    the dictionary. Removal listeners can be:

    - *eject only* listeners, meaning they're only notified when objects are
      ejected from the cache to make room for new objects, or
    - *removal* listeners, meaning they're notified whenever an object is
      removed for *any* reason, including via ``del``.

    This implementation is based on a Java ``LRUMap`` class in the
    ``org.clapper.util`` library. See http://www.clapper.org/software/java/util/
    for details.
    """
    def __init__(self, *args, **kw):
        """
        Initialize an ``LRUDict`` that will hold, at most, ``max_capacity``
        items. Attempts to insert more than ``max_capacity`` items in the
        dictionary will cause the least-recently used entries to drop out of
        the dictionary.

        :Keywords:
            max_capacity : int
                The maximum size of the dictionary
        """
        if kw.has_key('max_capacity'):
            self.__max_capacity = kw['max_capacity']
            del kw['max_capacity']
        else:
            self.__max_capacity = sys.maxint
            
        dict.__init__(self)
        self.__removal_listeners = {}
        self.__lru_queue = LRUList()
        
    def __del__(self):
        self.clear()

    def get_max_capacity(self):
        """
        Get the maximum capacity of the dictionary.
        
        :rtype: int
        :return: the maximum capacity
        """
        return self.__max_capacity

    def set_max_capacity(self, new_capacity):
        """
        Set or change the maximum capacity of the dictionary. Reducing
        the size of a dictionary with items already in it might result
        in items being evicted.
        
        :Parameters:
            new_capacity : int
                the new maximum capacity
        """
        self.__max_capacity = new_capacity
        if len(self) > new_capacity:
            self.__clear_to(new_capacity)

    max_capacity = property(get_max_capacity, set_max_capacity,
                            doc='The maximum capacity. Can be reset at will.')

    def add_ejection_listener(self, listener, *args):
        """
        Add an ejection listener to the dictionary. The listener function
        should take at least two parameters: the key and value being removed.
        It can also take additional parameters, which are passed through
        unmodified.

        An ejection listener is only notified when objects are ejected from
        the cache to make room for new objects; more to the point, an ejection
        listener is never notified when an object is removed from the cache
        manually, via use of the ``del`` operator.

        :Parameters:
            listener : function
                Function to invoke

            args : iterable
                Any additional parameters to pass to the function
        """
        self.__removal_listeners[listener] = (True, args)

    def add_removal_listener(self, listener, *args):
        """
        Add a removal listener to the dictionary. The listener function should
        take at least two parameters: the key and value being removed. It can
        also take additional parameters, which are passed through unmodified.

        A removal listener is notified when objects are ejected from the cache
        to make room for new objects *and* when objects are manually deleted
        from the cache.

        :Parameters:
            listener : function
                Function to invoke

            args : iterable
                Any additional parameters to pass to the function
        """
        self.__removal_listeners[listener] = (False, args)

    def remove_listener(self, listener):
        """
        Remove the specified removal or ejection listener from the list of
        listeners.

        :Parameters:
            listener : function
                Function object to remove

        :rtype: bool
        :return: ``True`` if the listener was found and removed; ``False``
                 otherwise
        """
        try:
            del self.__removal_listeners[listener]
            return True
        except KeyError:
            return False
        
    def clear_listeners(self):
        """
        Clear all removal and ejection listeners from the list of listeners.
        """
        for key in self.__removal_listeners.keys():
            del self.__removal_listeners[key]

    def __setitem__(self, key, value):
        self.__put(key, value)
        
    def __getitem__(self, key):
        lru_entry = dict.__getitem__(self, key)
        self.__lru_queue.move_to_head(lru_entry)
        return lru_entry.value

    def __delitem__(self, key):
        lru_entry = dict.__getitem__(self, key)
        self.__lru_queue.remove(lru_entry)
        dict.__delitem__(self, key)
        self.__notify_listeners(False, [(lru_entry.key, lru_entry.value)])

    def __str__(self):
        s = '{'
        sep = ''
        for k, v in self.iteritems():
            s += sep
            if type(k) == str:
                s += "'%s'" % k
            else:
                s += str(k)

            s += ': '
            if type(v) == str:
                s += "'%s'" % v
            else:
                s += str(v)
            sep = ', '
        s += '}'
        return s

    def __iter__(self):
        return self.__lru_queue.__iter__()

    def clear(self):
        self.__clear_to(0)

    def get(self, key, default=None):
        try:
            lru_entry = self.__getitem__(key)
            value = lru_entry.value
        except KeyError:
            value = default
        return value

    def keys(self):
        return self.__lru_queue.keys()

    def items(self):
        return self.__lru_queue.items()

    def values(self):
        return self.__lru_queue.values()

    def iteritems(self):
        return self.__lru_queue.iteritems()

    def iterkeys(self):
        return self.__lru_queue.iterkeys()

    def itervalues(self):
        return self.__lru_queue.itervalues()

    def update(self, d):
        for key, value in d.iteritems():
            self[key] = value

    def pop(self, key, default=None):
        try:
            result = self[key]
            del self[key]

        except KeyError:
            if not default:
                raise

            result = default

        return result

    def popitem(self):
        """
        Pops the least recently used recent key/value pair from the
        dictionary.
        
        :rtype: tuple
        :return: the least recent key/value pair, as a tuple
        
        :raise KeyError: empty dictionary
        """
        if len(self) == 0:
            raise KeyError, 'Attempted popitem() on empty dictionary'

        lru_entry = self.__lru_queue.remove_tail()
        dict.__delitem__(self, lru_entry.key)
        return lru_entry.key, lru_entry.value

    def __put(self, key, value):
        try:
            lru_entry = dict.__getitem__(self, key)

            # Replacing an existing value with a new one. Move the entry
            # to the head of the list.

            lru_entry.value = value
            self.__lru_queue.move_to_head(lru_entry)

        except KeyError:
            # Not there. Have to add a new one. Clear out the cruft first.
            # Preserve one of the entries we're clearing, to avoid
            # reallocation.

            lru_entry = self.__clear_to(self.max_capacity - 1)
            if lru_entry:
                lru_entry.key, lru_entry.value = key, value
            else:
                lru_entry = LRUListEntry(key, value)
            self.__lru_queue.add_to_head(lru_entry)

        dict.__setitem__(self, key, lru_entry)

    def __clear_to(self, size):
        old_tail = None
        while len(self.__lru_queue) > size:
            old_tail = self.__lru_queue.remove_tail()
            assert old_tail
            key = old_tail.key
            value = dict.__delitem__(self, key)
            self.__notify_listeners(True, [(key, value)])

        assert len(self.__lru_queue) <= size
        assert len(self) == len(self.__lru_queue)
        return old_tail

    def __notify_listeners(self, ejecting, key_value_pairs):
        if self.__removal_listeners:
            for key, value in key_value_pairs:
                for func, func_data in self.__removal_listeners.items():
                    on_eject_only, args = func_data
                    if (not on_eject_only) or ejecting:
                        func(key, value, *args)



# $Id$

"""
Tester.
"""

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

from timeit import Timer

from grizzled.collections import LRUDict

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class TestLRUDict(object):

    def test1(self):
        lru = LRUDict(max_capacity=5)

        print "Adding 'a' and 'b'"
        lru['a'] = 'A'
        lru['b'] = 'b'
        print lru
        print lru.keys()
        assert lru.keys() == ['b', 'a']
        assert lru.values() == ['b', 'A']

        print "Adding 'c'"
        lru['c'] = 'c'
        print lru
        print lru.keys()
        assert lru.keys() == ['c', 'b', 'a']

        print "Updating 'a'"
        lru['a'] = 'a'
        print lru
        print lru.keys()
        assert lru.keys() == ['a', 'c', 'b']

        print "Adding 'd' and 'e'"
        lru['d'] = 'd'
        lru['e'] = 'e'
        print lru
        print lru.keys()
        assert lru.keys() == ['e', 'd', 'a', 'c', 'b']

        print "Accessing 'b'"
        assert lru['b'] == 'b'
        print lru
        print lru.keys()
        assert lru.keys() == ['b', 'e', 'd', 'a', 'c']

        print "Adding 'f'"
        lru['f'] = 'f'
        # Should knock 'c' out of the list
        print lru
        print lru.keys()
        assert lru.keys() == ['f', 'b', 'e', 'd', 'a']
        
        def on_remove(key, value, the_list):
            print 'on_remove("%s")' % key
            the_list.append(key)

        print 'Reducing capacity. Should result in eviction.'
        ejected = []
        lru.add_ejection_listener(on_remove, ejected)
        lru.max_capacity = 3
        ejected.sort()
        print 'ejected=%s' % ejected
        assert ejected == ['a', 'd']
        print lru.keys()
        assert lru.keys() == ['f', 'b', 'e']
        
        print 'Testing popitem()'
        key, value = lru.popitem()
        print lru
        print lru.keys()
        assert key == 'e'
        assert lru.keys() == ['f', 'b']

        print 'Clearing dictionary'
        lru.clear_listeners()
        lru.clear()
        del lru

    def add_one(self, lru, key):
        lru[key] = key

    def testBig(self):
        print 'Putting 10000 entries in a new LRU cache'
        lru = LRUDict(max_capacity=10000)
        for i in range(0, lru.max_capacity):
            lru[i] = i

        assert len(lru) == lru.max_capacity
        print 'Adding one more'
        assert len(lru) == lru.max_capacity
        print iter(lru).next()



"""
This module contains file- and path-related methods, classes, and modules.
"""

from __future__ import with_statement, absolute_import

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import os as _os
import sys
import shutil

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['unlink_quietly', 'recursively_remove', 'copy_recursively',
           'copy', 'touch', 'pathsplit', 'eglob', 'universal_path',
           'native_path']

# ---------------------------------------------------------------------------
# Functions
# ---------------------------------------------------------------------------

def unlink_quietly(*paths):
    """
    Like the standard ``os.unlink()`` function, this function attempts to
    delete a file. However, it swallows any exceptions that occur during the
    unlink operation, making it more suitable for certain uses (e.g.,
    in ``atexit`` handlers).

    :Parameters:
        paths : str or list
            path(s) to unlink
    """
    def looper(*paths):
        for i in paths:
            if type(i) == list:
                for path in i:
                    yield path
            else:
                yield i

    for path in looper(*paths):
        try:
            _os.unlink(path)
        except:
            pass

def recursively_remove(dir):
    """
    Recursively remove all files and directories below and including a
    specified directory.

    :Parameters:
        dir : str
            path to directory to remove
    """
    if not _os.path.exists(dir):
        return

    shutil.rmtree(dir)

def list_recursively(dir):
    """
    Recursively list the contents of a directory. Yields the contents of
    the directory and all subdirectories. This method returns a generator,
    so it evaluates its recursive walk lazily.

    :Parameters:
        dir : str
            Path to directory to list

    :raise ValueError: If ``dir`` does not exist, or if ``dir`` exists
                       but is not a directory.
    """
    if not _os.path.isdir(dir):
        raise ValueError, "%s is not a directory." % dir

    for f in _os.listdir(dir):
        if _os.path.isdir(f):
            list_recursively(f)
        else:
            yield f

def copy_recursively(source_dir, target_dir):
    """
    Recursively copy a source directory (and all its contents) to a target
    directory.

    :Parameters:
        source_dir : str
            Source directory to copy recursively. This path must
            exist and must specify a directory; otherwise, this
            function throws a ``ValueError``

        target_dir : str
            Directory to which to copy the contents of ``source_dir``.
            This directory must not already exist.

    :raise ValueError: If: ``source_dir`` does not exist; ``source_dir`` exists
                       but is not a directory; or ``target_dir`` exists but is
                       not a directory.
    """
    shutil.copytree(source_dir, target_dir)

def copy(files, target_dir, create_target=False):
    """
    Copy one or more files to a target directory.

    :Parameters:
        files : str or list
            single file path or a list of file paths to be copied

        target_dir : str
            path to target directory

        create_target : bool
            If ``True``, ``copy()`` will attempt to create the target directory
            if it does not exist. If ``False``, ``copy()`` will throw an
            exception if the target directory does not exist.
            
    :raise OSError: ``target_dir`` does not exist, and ``create_target`` is
                    ``False``
    """
    if type(files) == str:
        files = [files]

    if not _os.path.exists(target_dir):
        if create_target:
            _os.mkdir(target_dir)

    if _os.path.exists(target_dir) and (not _os.path.isdir(target_dir)):
        raise OSError, 'Cannot copy files to non-directory "%s"' % target_dir

    for f in files:
        targetFile = _os.path.join(target_dir, _os.path.basename(f))
        open(targetFile, 'wb').write(open(f, 'rb').read())

def touch(files, times=None):
    """
    Similar to the Unix *touch* command, this function:

    - updates the access and modification times for any existing files
      in a list of files
    - creates any non-existent files in the list of files

    If any file in the list is a directory, this function will throw an
    exception.

    :Parameters:
        files : list or str
            pathname or list of pathnames of files to be created or updated

        times : tuple
            tuple of the form (*atime*, *mtime*), identical to
            what is passed to the standard ``os.utime()`` function.
            If this tuple is ``None``, then the current time is used.
    """
    if type(files) == str:
        files = [files]

    for f in files:
        if _os.path.exists(f):
            if not _os.path.isfile(f):
                raise OSError, "Can't touch non-file \"%s\"" % f
            _os.utime(f, times)

        else:
            # Doesn't exist. Create it.
            open(f, 'wb').close()


def pathsplit(path):
    """
    Split a path into an array of path components, using the file separator
    ('/' on POSIX systems, '\' on Windows) that's appropriate for the
    underlying operating system. Does not take drive letters into account.
    If there's a Windows drive letter in the path, it'll end up with the
    first component.

    :Parameters:
        path : str
            path to split. Can be relative or absolute

    :rtype:  list
    :return: a list of path components
    """
    result = []
    (head, tail) = _os.path.split(path)

    if (not head) or (head == path):
        # No file separator. Done.
        pass

    else:
        result = pathsplit(head)

    if tail:
        result += [tail]

    return result

def __find_matches(pattern_pieces, directory):
    """
    Used by eglob.
    """
    import glob

    result = []
    if not _os.path.isdir(directory):
        return []

    piece = pattern_pieces[0]
    last = len(pattern_pieces) == 1
    if piece == '**':
        if not last:
            remaining_pieces = pattern_pieces[1:]

        for root, dirs, files in _os.walk(directory):
            if last:
                # At the end of a pattern, "**" just recursively matches
                # directories.
                result += [root]
            else:
                # Recurse downward, trying to match the rest of the
                # pattern.
                sub_result = __find_matches(remaining_pieces, root)
                for partial_path in sub_result:
                    result += [partial_path]

    else:
        # Regular glob pattern.

        matches = glob.glob(_os.path.join(directory, piece))
        if len(matches) > 0:
            if last:
                for match in matches:
                    result += [match]
            else:
                remaining_pieces = pattern_pieces[1:]
                for match in matches:
                    sub_result = __find_matches(remaining_pieces, match)
                    for partial_path in sub_result:
                        result += [partial_path]

    # Normalize the paths.

    for i in range(len(result)):
        result[i] = _os.path.normpath(result[i])

    return result

def eglob(pattern, directory='.'):
    """
    Extended glob function that supports the all the wildcards supported
    by the Python standard ``glob`` routine, as well as a special "**"
    wildcard that recursively matches any directory. Examples:
    
      +--------------+--------------------------------------------------------+
      | \*\*/\*.py   | all files ending in '.py' under the current directory  |
      +--------------+--------------------------------------------------------+
      | foo/\*\*/bar | all files name 'bar' anywhere under subdirectory 'foo' |
      +--------------+--------------------------------------------------------+

    :Parameters:
        pattern : str
            The wildcard pattern. Must be a simple pattern with no directories.

        directory : str
            The directory in which to do the globbing.

    :rtype:  list
    :return: A list of matched files, or an empty list for no match
    """
    pieces = pathsplit(pattern)
    return __find_matches(pieces, directory)

def universal_path(path):
    """
    Converts a path name from its operating system-specific format to a
    universal path notation. Universal path notation always uses a Unix-style
    "/" to separate path elements. A universal path can be converted to a
    native (operating system-specific) path via the ``native_path()``
    function. Note that on POSIX-compliant systems, this function simply
    returns the ``path`` parameter unmodified.

    :Parameters:
        path : str
            the path to convert to universal path notation

    :rtype:  str
    :return: the universal path.
    """
    if _os.name != 'posix':
        path = path.replace(_os.path.sep, '/')

    return path

def native_path(path):
    """
    Converts a path name from universal path notation to the operating
    system-specific format. Universal path notation always uses a Unix-style
    "/" to separate path elements. A native path can be converted to a
    universal path via the ``universal_path()`` function. Note that on
    POSIX-compliant systems, this function simply returns the ``path``
    parameter unmodified.

    :Parameters:
        path : str
            the path to convert to native path notation

    :rtype:  str
    :return: the native path.
    """
    if _os.name != 'posix':
        path = path.replace('/', _os.path.sep)

    return path



# $Id: 38c71003e81b9ea1df83aefcb90fb198cc8a44ef $

"""
Base classes for enhanced DB drivers.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import re
import time
import os
import sys
from datetime import date, datetime

from grizzled.exception import ExceptionWithMessage
from grizzled.decorators import abstract
from grizzled.collections import namedtuple

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['DBDriver', 'DB', 'Cursor', 'DBError', 'Error', 'Warning',
           'TableMetadata', 'IndexMetadata', 'RDBMSMetadata']

# ---------------------------------------------------------------------------
# Globals
# ---------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class DBError(ExceptionWithMessage):
    """
    Base class for all DB exceptions.
    """
    pass

class Error(DBError):
    """Thrown to indicate an error in the ``db`` module."""
    pass

class Warning(DBError):
    """Thrown to indicate an error in the ``db`` module."""
    pass

TableMetadata = namedtuple('TableMetadata', ['column_name',
                                             'type_string',
                                             'max_char_size',
                                             'precision',
                                             'scale',
                                             'nullable'])

IndexMetadata = namedtuple('IndexMetadata', ['index_name',
                                             'index_columns',
                                             'description'])

RDBMSMetadata = namedtuple('RDBMSMetadata', ['vendor', 'product', 'version'])

class Cursor(object):
    """
    Class for DB cursors returned by the ``DB.cursor()`` method. This class
    conforms to the Python DB cursor interface, including the following
    attributes.

    :IVariables:
        description : tuple
            A read-only attribute that is a sequence of 7-item tuples, one per
            column, from the last query executed. The tuple values are:
            *(name, typecode, displaysize, internalsize, precision, scale)*
        rowcount : int
            A read-only attribute that specifies the number of rows
            fetched in the last query, or -1 if unknown. *Note*: It's best
            not to rely on the row count, because some database drivers
            (such as SQLite) don't report valid row counts.
    """

    def __init__(self, cursor, driver):
        """
        Create a new Cursor object, wrapping the underlying real DB API
        cursor.

        :Parameters:
            cursor
                the real DB API cursor object
            driver
                the driver that is creating this object
        """
        self.__cursor = cursor
        self.__driver = driver
        self.__description = None
        self.__rowcount = -1

    def __get_description(self):
        return self.__description

    description = property(__get_description,
                           doc='The description field. See class docs.')

    def __get_rowcount(self):
        return self.__rowcount

    rowcount = property(__get_rowcount,
                        doc='Number of rows from last query, or -1')

    def close(self):
        """
        Close the cursor.

        :raise Warning: Non-fatal warning
        :raise Error:   Error; unable to close
        """
        dbi = self.__driver.get_import()
        try:
            return self.__cursor.close()
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    def execute(self, statement, parameters=None):
        """
        Execute a SQL statement string with the given parameters.
        'parameters' is a sequence when the parameter style is
        'format', 'numeric' or 'qmark', and a dictionary when the
        style is 'pyformat' or 'named'. See ``DB.paramstyle()``.

        :Parameters:
            statement : str
                the SQL statement to execute
            parameters : list
                parameters to use, if the statement is parameterized

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.__driver.get_import()
        try:
            if parameters:
                result = self.__cursor.execute(statement, parameters)
            else:
                result = self.__cursor.execute(statement)

            try:
                self.__rowcount = self.__cursor.rowcount
            except AttributeError:
                self.__rowcount = -1
            self.__description = self.__cursor.description
            return result
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)
        except:
            raise Error(sys.exc_info()[1])

    def executemany(self, statement, *parameters):
        """
        Execute a SQL statement once for each item in the given parameters.

        :Parameters:
            statement : str
                the SQL statement to execute
            parameters : sequence
                a sequence of sequences when the parameter style
                is 'format', 'numeric' or 'qmark', and a sequence
                of dictionaries when the style is 'pyformat' or
                'named'.

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.__driver.get_import()
        try:
            result = self.__cursor.executemany(statement, *parameters)
            self.__rowcount = self.__cursor.rowcount
            self.__description = self.__cursor.description
            return result
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    executeMany = executemany

    def fetchone(self):
        """
        Returns the next result set row from the last query, as a sequence
        of tuples. Raises an exception if the last statement was not a query.

        :rtype:  tuple
        :return: Next result set row

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.__driver.get_import()
        try:
            return self.__cursor.fetchone()
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    def fetchall(self):
        """
        Returns all remaining result rows from the last query, as a sequence
        of tuples. Raises an exception if the last statement was not a query.

        :rtype:  list of tuples
        :return: List of rows, each represented as a tuple

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.__driver.get_import()
        try:
            return self.__cursor.fetchall()
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    fetchAll = fetchall

    def fetchmany(self, n):
        """
        Returns up to n remaining result rows from the last query, as a
        sequence of tuples. Raises an exception if the last statement was
        not a query.

        :Parameters:
            n : int
                maximum number of result rows to get

        :rtype:  list of tuples
        :return: List of rows, each represented as a tuple

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.__driver.get_import()
        try:
            self.__cursor.fetchmany(n)
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    fetchMany = fetchmany

    def get_rdbms_metadata(self):
        """
        Return data about the RDBMS: the product name, the version,
        etc. The result is a named tuple, with the following fields:

        vendor
            The product vendor, if applicable, or ``None`` if not known
        product
            The name of the database product, or ``None`` if not known
        version
            The database product version, or ``None`` if not known

        The fields may be accessed by position or name. This method
        just calls through to the equivalent method in the underlying
        ``DBDriver`` implementation.

        :rtype: named tuple
        :return: the vendor information
        """
        # Default implementation
        dbi = self.__driver.get_import()
        try:
            return self.__driver.get_rdbms_metadata(self.__cursor)
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    def get_table_metadata(self, table):
        """
        Get the metadata for a table. Returns a list of tuples, one for
        each column. Each tuple consists of the following::

            (column_name, type_string, max_char_size, precision, scale, nullable)

        The tuple elements have the following meanings.

        column_name
            the name of the column
        type_string
            the column type, as a string
        max_char_size
            the maximum size for a character field, or ``None``
        precision
            the precision, for a numeric field; or ``None``
        scale
            the scale, for a numeric field; or ``None``
        nullable
            True if the column is nullable, False if it is not

        The tuples are named tuples, so the fields may be referenced by the
        names above or by position.

        The data may come from the DB API's ``cursor.description`` field, or
        it may be richer, coming from a direct SELECT against
        database-specific tables.

        :rtype: list
        :return: list of tuples, as described above

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        # Default implementation
        dbi = self.__driver.get_import()
        try:
            return self.__driver.get_table_metadata(table, self.__cursor)
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    def get_index_metadata(self, table):
        """
        Get the metadata for the indexes for a table. Returns a list of
        tuples, one for each index. Each tuple consists of the following::

            (index_name, [index_columns], description)

        The tuple elements have the following meanings.

        index_name
            the index name
        index_columns
            a list of column names
        description
            index description, or ``None``

        The tuples are named tuples, so the fields may be referenced by the
        names above or by position.

        :rtype:  list of tuples
        :return: the list of tuples, or ``None`` if not supported in the
                 underlying database

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.__driver.get_import()
        try:
            return self.__driver.get_index_metadata(table, self.__cursor)
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    def get_tables(self):
        """
        Get the list of tables in the database to which this cursor is
        connected.

        :rtype:  list
        :return: List of table names. The list will be empty if the database
                 contains no tables.

        :raise NotImplementedError: Capability not supported by database driver
        :raise Warning:             Non-fatal warning
        :raise Error:               Error
        """
        dbi = self.__driver.get_import()
        try:
            return self.__driver.get_tables(self.__cursor)
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

class DB(object):
    """
    The object returned by a call to ``DBDriver.connect()``. ``db`` wraps the
    real database object returned by the underlying Python DB API module's
    ``connect()`` method.
    """
    def __init__(self, db, driver):
        """
        Create a new DB object.

        :Parameters:
            db
                the underlying Python DB API database object
            driver : DBDriver
                the driver (i.e., the subclass of ``DBDriver``) that
                created the ``db`` object
        """
        self.__db = db
        self.__driver = driver
        dbi = driver.get_import()
        for attr in ['BINARY', 'NUMBER', 'STRING', 'DATETIME', 'ROWID']:
            try:
                exec 'self.%s = dbi.%s' % (attr, attr)
            except AttributeError:
                exec 'self.%s = 0' % attr

    def paramstyle(self):
        """
        Get the parameter style for the underlying DB API module. The
        result of this method call corresponds exactly to the underlying
        DB API module's 'paramstyle' attribute. It will have one of the
        following values:

        +----------+-----------------------------------------------------------+
        | format   | The parameter marker is '%s', as in string                |
        |          | formatting. A query looks like this::                     |
        |          |                                                           |
        |          |   c.execute('SELECT * FROM Foo WHERE Bar=%s', [x])        |
        +----------+-----------------------------------------------------------+
        | named    | The parameter marker is ``:name``, and parameters         |
        |          | are named. A query looks like this::                      |
        |          |                                                           |
        |          |   c.execute('SELECT * FROM Foo WHERE Bar=:x', {'x':x})    |
        +----------+-----------------------------------------------------------+
        | numeric  | The parameter marker is ``:n``, giving the parameter's    |
        |          | number (starting at 1). A query looks like this::         |
        |          |                                                           |
        |          |   c.execute('SELECT * FROM Foo WHERE Bar=:1', [x])        |
        +----------+-----------------------------------------------------------+
        | pyformat | The parameter marker is ``:name``, and parameters         |
        |          | are named. A query looks like this::                      |
        |          |                                                           |
        |          |   c.execute('SELECT * FROM Foo WHERE Bar=%(x)s', {'x':x}) |
        +----------+-----------------------------------------------------------+
        | qmark    | The parameter marker is "?", and parameters are           |
        |          | substituted in order. A query looks like this::           |
        |          |                                                           |
        |          |   c.execute('SELECT * FROM Foo WHERE Bar=?', [x])         |
        +----------+-----------------------------------------------------------+
        """
        return self.__driver.get_import().paramstyle

    def Binary(self, string):
        """
        Returns an object representing the given string of bytes as a BLOB.

        This method is equivalent to the module-level ``Binary()`` method in
        an underlying DB API-compliant module.

        :Parameters:
            string : str
                the string to convert to a BLOB

        :rtype:  object
        :return: the corresponding BLOB
        """
        return self.__driver.get_import().Binary(string)

    def Date(self, year, month, day):
        """
        Returns an object representing the specified date.

        This method is equivalent to the module-level ``Date()`` method in
        an underlying DB API-compliant module.

        :Parameters:
            year
                the year
            month
                the month
            day
                the day of the month

        :return: an object containing the date
        """
        return self.__driver.get_import().Date(year, month, day)

    def DateFromTicks(self, secs):
        """
        Returns an object representing the date *secs* seconds after the
        epoch. For example:

        .. python::

            import time

            d = db.DateFromTicks(time.time())

        This method is equivalent to the module-level ``DateFromTicks()``
        method in an underlying DB API-compliant module.

        :Parameters:
            secs : int
                the seconds from the epoch

        :return: an object containing the date
        """
        date = date.fromtimestamp(secs)
        return self.__driver.get_import().Date(date.year, date.month, date.day)

    def Time(self, hour, minute, second):
        """
        Returns an object representing the specified time.

        This method is equivalent to the module-level ``Time()`` method in an
        underlying DB API-compliant module.

        :Parameters:
            hour
                the hour of the day
            minute
                the minute within the hour. 0 <= *minute* <= 59
            second
                the second within the minute. 0 <= *second* <= 59

        :return: an object containing the time
        """
        dt = datetime.fromtimestamp(secs)
        return self.__driver.get_import().Time(dt.hour, dt.minute, dt.second)

    def TimeFromTicks(self, secs):
        """
        Returns an object representing the time 'secs' seconds after the
        epoch. For example:

        .. python::

            import time

            d = db.TimeFromTicks(time.time())

        This method is equivalent to the module-level ``TimeFromTicks()``
        method in an underlying DB API-compliant module.

        :Parameters:
            secs : int
                the seconds from the epoch

        :return: an object containing the time
        """
        dt = datetime.fromtimestamp(secs)
        return self.__driver.get_import().Time(dt.hour, dt.minute, dt.second)

    def Timestamp(self, year, month, day, hour, minute, second):
        """
        Returns an object representing the specified time.

        This method is equivalent to the module-level ``Timestamp()`` method
        in an underlying DB API-compliant module.

        :Parameters:
            year
                the year
            month
                the month
            day
                the day of the month
            hour
                the hour of the day
            minute
                the minute within the hour. 0 <= *minute* <= 59
            second
                the second within the minute. 0 <= *second* <= 59

        :return: an object containing the timestamp
        """
        return self.__driver.get_import().Timestamp(year, month, day,
                                                    hour, minute, second)

    def TimestampFromTicks(self, secs):
        """
        Returns an object representing the date and time ``secs`` seconds
        after the epoch. For example:

        .. python::

            import time

            d = db.TimestampFromTicks(time.time())

        This method is equivalent to the module-level ``TimestampFromTicks()``
        method in an underlying DB API-compliant module.

        :Parameters:
            secs : int
                the seconds from the epoch

        :return: an object containing the timestamp
        """
        dt = datetime.now()
        return self.__driver.get_import().Timestamp(dt.year, dt.month, dt.day,
                                                    dt.hour, dt.minute, dt.second)

    def cursor(self):
        """
        Get a cursor suitable for accessing the database. The returned object
        conforms to the Python DB API cursor interface.

        :return: the cursor

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.__driver.get_import()
        try:
            return Cursor(self.__db.cursor(), self.__driver)
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    def commit(self):
        """
        Commit the current transaction.

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.__driver.get_import()
        try:
            self.__db.commit()
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    def rollback(self):
        """
        Roll the current transaction back.

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.__driver.get_import()
        try:
            self.__db.rollback()
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    def close(self):
        """
        Close the database connection.

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.__driver.get_import()
        try:
            self.__db.close()
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

class DBDriver(object):
    """
    Base class for all DB drivers.
    """

    @abstract
    def get_import(self):
        """
        Get a bound import for the underlying DB API module. All subclasses
        must provide an implementation of this method. Here's an example,
        assuming the real underlying Python DB API module is 'foosql':

        .. python::

            def get_import(self):
                import foosql
                return foosql

        :return: a bound module
        """
        pass

    def __display_name(self):
        return self.get_display_name()

    @abstract
    def get_display_name(self):
        """
        Get the driver's name, for display. The returned name ought to be
        a reasonable identifier for the database (e.g., 'SQL Server',
        'MySQL'). All subclasses must provide an implementation of this
        method.

        :rtype:  str
        :return: the driver's displayable name
        """
        pass

    display_name = property(__display_name,
                            doc='get a displayable name for the driver')
    def connect(self,
                host='localhost',
                port=None,
                user=None,
                password='',
                database=None):
        """
        Connect to the underlying database. Subclasses should *not*
        override this method. Instead, a subclass should override the
        ``do_connect()`` method.

        :Parameters:
            host : str
                the host where the database lives
            port : int
                the TCP port to use when connecting, or ``None``
            user : str
                the user to use when connecting, or ``None``
            password : str
                the password to use when connecting, or ``None``
            database : str
                the name of the database to which to connect

        :rtype:  ``db``
        :return: a ``db`` object representing the open database

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        dbi = self.get_import()
        try:
            self.__db = self.do_connect(host=host,
                                       port=port,
                                       user=user,
                                       password=password,
                                       database=database)
            return DB(self.__db, self)
        except dbi.Warning, val:
            raise Warning(val)
        except dbi.Error, val:
            raise Error(val)

    @abstract
    def do_connect(self,
                   host='localhost',
                   port=None,
                   user='',
                   password='',
                   database='default'):
        """
        Connect to the actual underlying database, using the driver.
        Subclasses must provide an implementation of this method. The
        method must return the result of the real DB API implementation's
        ``connect()`` method. For instance:

        .. python::

            def do_connect():
                dbi = self.get_import()
                return dbi.connect(host=host, user=user, passwd=password,
                                   database=database)

        There is no need to catch exceptions; the ``DBDriver`` class's
        ``connect()`` method handles that.

        :Parameters:
            host : str
                the host where the database lives
            port : int
                the TCP port to use when connecting
            user : str
                the user to use when connecting
            password : str
                the password to use when connecting
            database : str
                the name of the database to which to connect

        :rtype:  object
        :return: a DB API-compliant object representing the open database

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        pass

    def get_rdbms_metadata(self, cursor):
        """
        Return data about the RDBMS: the product name, the version,
        etc. The result is a named tuple, with the following fields.

        vendor
            The product vendor, if applicable, or ``None`` if not known
        product
            The name of the database product, or ``None`` if not known
        version
            The database product version, or ``None`` if not known

        :Parameters:
            cursor : Cursor
                a ``Cursor`` object from a recent query

        :rtype: named tuple
        :return: the vendor information
        """
        return RDBMSMetadata('unknown', 'unknown', 'unknown')

    def get_index_metadata(self, table, cursor):
        """
        Get the metadata for the indexes for a table. Returns a list of
        tuples, one for each index. Each tuple consists of the following::

            (index_name, [index_columns], description)

        The tuple elements have the following meanings.

        index_name
            the index name
        index_columns
            a list of column names
        description
            index description, or ``None``

        The tuples are named tuples, so the fields may be referenced by the
        names above or by position.

        The default implementation of this method returns `None`

        :Parameters:
            table : str
                table name
            cursor : Cursor
                a ``Cursor`` object from a recent query

        :rtype:  list of tuples
        :return: the list of tuples, or ``None`` if not supported in the
                 underlying database

        :raise Warning: Non-fatal warning
        """
        return None

    def get_table_metadata(self, table, cursor):
        """
        Get the metadata for a table. Returns a list of tuples, one for
        each column. Each tuple consists of the following::

            (column_name, type_string, max_char_size, precision, scale, nullable)

        The tuple elements have the following meanings.

        column_name
            the name of the column
        type_string
            the column type, as a string
        max_char_size
            the maximum size for a character field, or ``None``
        precision
            the precision, for a numeric field; or ``None``
        scale
            the scale, for a numeric field; or ``None``
        nullable
            ``True`` if the column is nullable, ``False`` if it is not

        The tuples are named tuples, so the fields may be referenced by the
        names above or by position.

        The default implementation uses the DB API's ``cursor.description``
        field. Subclasses are free to override this method to produce their
        own version that uses other means.

        :Parameters:
            table : str
                the table name for which metadata is desired
            cursor : Cursor
                a ``Cursor`` object from a recent query

        :rtype: list
        :return: list of tuples, as described above

        :raise Warning: Non-fatal warning
        :raise Error:   Error
        """
        self._ensure_valid_table(cursor, table)
        dbi = self.get_import()
        cursor.execute('SELECT * FROM %s WHERE 1=0' % table)
        result = []
        for col in cursor.description:
            name = col[0]
            type = col[1]
            size = col[2]
            internalSize = col[3]
            precision = col[4]
            scale = col[5]
            nullable = col[6]

            sType = None
            try:
                if type == dbi.BINARY:
                    stype = 'blob'
                elif type == dbi.DATETIME:
                    stype = 'datetime'
                elif type == dbi.NUMBER:
                    stype = 'numeric'
                elif type == dbi.STRING:
                    sz = internalSize
                    if sz == None:
                        sz = size
                    elif sz <= 0:
                        sz = size

                    if sz == 1:
                        stype = 'char'
                    else:
                        stype = 'varchar'
                    size = sz
                elif type == dbi.ROWID:
                    stype = 'id'
            except AttributeError:
                stype = None

            if not sType:
                stype = 'unknown (type code=%s)' % str(type)

            data = TableMetadata(name, stype, size, precision, scale, nullable)
            result += [data]

        return result

    def get_tables(self, cursor):
        """
        Get the list of tables in the database.

        :Parameters:
            cursor : Cursor
                a ``Cursor`` object from a recent query

        :rtype:  list
        :return: List of table names. The list will be empty if the database
                 contains no tables.

        :raise NotImplementedError: Capability not supported by database driver
        :raise Warning:             Non-fatal warning
        :raise Error:               Error
        """
        raise NotImplementedError

    def _ensure_valid_table(self, cursor, table_name):
        """
        Determines whether a table name represents a legal table in the
        current database, throwing an ``Error`` if not.

        :Parameters:
            cursor : Cursor
                an open ``Cursor``

            table_name : str
                the table name

        :raise Error: bad table name
        """
        if not self._is_valid_table(cursor, table_name):
            raise Error, 'No such table: "%s"' % table_name

    def _is_valid_table(self, cursor, table_name):
        """
        Determines whether a table name represents a legal table in the
        current database, throwing an ``Error`` if not.

        :Parameters:
            cursor : Cursor
                an open ``Cursor``

            table_name : str
                the table name

        :rtype: bool
        :return: ``True`` if the table is valid, ``False`` if not
        """
        tables = self.get_tables(cursor)
        return table_name in tables



# NOTE: Documentation is intended to be processed by epydoc and contains
# epydoc markup.

"""
Overview
========

The ``grizzled.system`` module contains some functions and classes that
provide information about the Python system (the Python runtime, the language,
etc.). It is a conceptual extension of the standard Python ``sys`` module.
"""

from __future__ import absolute_import

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import sys as _sys
import logging
import re

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['python_version', 'python_version_string', 'ensure_version',
           'split_python_version', 'class_for_name']

# ---------------------------------------------------------------------------
# Constants
# ---------------------------------------------------------------------------

RELEASE_LEVEL_RE = re.compile(r'([0-9]+)(.[0-9]+)?')
RELEASE_LEVELS = {'a' : 0xa, 'b' : 0xb, 'c' : 0xc, 'f' : 0xf}
RELEASE_LEVEL_NAMES = {0xa : 'alpha',
                       0xb : 'beta',
                       0xc : 'candidate',
                       0xf : 'final'}

# ---------------------------------------------------------------------------
# Logging
# ---------------------------------------------------------------------------

log = logging.getLogger('grizzled.system')

# ---------------------------------------------------------------------------
# Public classes
# ---------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# Public functions
# ---------------------------------------------------------------------------

def python_version(version):
    """
    Convert a Python string version (e.g., "2.5.1", "1.3", "2.6a3") to a
    numeric version that can meaningfully be compared with the standard
    ``sys`` module's ``sys.hexversion`` value.

    For example, here's the usual way to ensure that your program is running
    under Python 2.5.1 or better:
    
    .. python::

        import sys

        if sys.hexversion < 0x020501f0:
            raise RuntimeError, 'This program requires Python 2.5.1 or better'

    Here's how you'd use ``python_version()`` to do the same thing (in an
    arguably more readable way):

    .. python::

        import sys
        from grizzled.sys import python_version

        if sys.hexversion < python_version("2.5.1"):
            raise RuntimeError, 'This program requires Python 2.5.1 or better'

    :Parameters:
        version : str
            string Python version to convert to binary

    :rtype:  int
    :return: corresponding integer Python version

    :raise ValueError: ``version`` isn't of the form "x", or "x.y" or
                       "x.y.z"
    """
    err = 'Malformed Python version "%s"' % version

    tokens = version.split('.')
    if len(tokens) > 3:
        raise ValueError, err

    major = int(tokens[0])
    minor = micro = serial = 0
    release_level = 'f'

    if len(tokens) > 1:
        match = RELEASE_LEVEL_RE.match(tokens[1])
        if not match:
            raise ValueError, err

        minor = int(match.group(1))
        rl = match.group(2)
        if rl:
            release_level = rl[0]
            serial = int(rl[1:])

        if len(tokens) > 2:
            match = RELEASE_LEVEL_RE.match(tokens[2])
            if not match:
                raise ValueError, err

            micro = int(match.group(1))
            rl2 = match.group(2)
            if rl and rl2:
                raise ValueError, err
            if rl2:
                release_level = rl2[0]
                serial = int(rl2[1:])

    try:
        release_level = RELEASE_LEVELS[release_level]
    except KeyError:
        raise ValueError, err

    return (major << 24) |\
           (minor << 16) |\
           (micro << 8) |\
           (release_level << 4) |\
           serial

def split_python_version(version=None):
    """
    Convert a binary Python version string (e.g., ``0x020501f0``) into the
    same (*major*, *minor*, *micro*, *releaselevel*, *serial*) tuple that is
    found in ``sys.version_info``. Thus, for an input value of ``0x020501f0``,
    this function returns the tuple ``(2, 5, 1, 'final', 0)``.

    :Parameters:
        version : int
            Python integer version

    :rtype:  tuple
    :return: The (*major*, *minor*, *micro*, *releaselevel*, *serial*) tuple

    :raise ValueError: Bad version number
    """
    major = (version >> 24) & 0x000000ff
    minor = (version >> 16) & 0x000000ff
    micro = (version >> 8) & 0x000000ff
    release_level = (version >> 4) & 0x0000000f
    serial = version & 0x0000000f

    release_level_string = RELEASE_LEVEL_NAMES.get(release_level, None)
    if not release_level_string:
        raise ValueError, \
              'Bad release level 0x%x in version 0x%08x' %\
              (release_level, version)

    return (major, minor, micro, release_level_string, serial)

def python_version_string(version=None):
    """
    Convert a numeric Python version (such as ``sys.hexversion``) to a
    printable string.

    :Parameters:
        version : int
            Python integer version

    :rtype:  str
    :return: The stringified version
    """
    major, minor, micro, release_level, serial = split_python_version(version)
    s = '%d.%d' % (major, minor)
    if micro > 0:
        s += '.%d' % micro

    if release_level != 'final':
        s += release_level[0]
        s += '%s' % serial

    return s

def ensure_version(min_version):
    """
    Raise a ``RuntimeError`` if the current Python version isn't at least
    ``min_version``. ``min_version`` may be an ``int`` (e.g., ``0x020500f0``)
    or a string (e.g., "2.5.0").

    :Parameters:
        min_version : str or int
            minimum version, as a number or string

    :raise TypeError:    ``min_version`` isn't a string or an integer
    :raise ValueError:   ``min_version`` is a bad Python string version
    :raise RuntimeError: Python version is too old
    """
    if type(min_version) == str:
        min_version = python_version(min_version)
    elif type(min_version) == int:
        pass
    else:
        raise TypeError, \
              'version %s is not a string or an integer' % min_version

    if _sys.hexversion < min_version:
        raise RuntimeError, \
              'This program requires Python version "%s" or better, but ' \
              'the current Python version is "%s".' %\
              (python_version_string(min_version),
               python_version_string(sys.hexversion))


def class_for_name(class_name):
    """
    Given fully-qualified class name, load and return the class object. A
    fully-qualified class name contains the module and package, in addition to
    the simple class name (e.g., ``grizzled.config.Configuration``).

    :Parameters:
        class_name : str
            fully-qualified class name

    :rtype:  class
    :return: the class object

    :raise NameError: Class not found
    """
    tokens = class_name.split('.')
    if len(tokens) > 1:
        package = '.'.join(tokens[:-1])
        class_name = tokens[-1]
        exec 'from %s import %s' % (package, class_name)

    return eval(class_name)
    



# $Id: 09ac132a27ad58c65e081c918f091fcfcc28b392 $

"""
MySQL extended database driver.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import os
import sys
import re

from grizzled.db.base import (DBDriver, Error, Warning, TableMetadata,
                              IndexMetadata, RDBMSMetadata)

# ---------------------------------------------------------------------------
# Constants
# ---------------------------------------------------------------------------

VENDOR  = 'MySQL AB'
PRODUCT = 'MySQL'

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class MySQLDriver(DBDriver):
    """DB Driver for MySQL, using the MySQLdb DB API module."""

    TYPE_RE = re.compile('([a-z]+)(\([0-9]+\))?')

    def get_import(self):
        import MySQLdb
        return MySQLdb

    def get_display_name(self):
        return "MySQL"

    def do_connect(self,
                   host="localhost",
                   port=None,
                   user="sa",
                   password="",
                   database="default"):
        dbi = self.get_import()
        return dbi.connect(host=host, user=user, passwd=password, db=database)

    def get_rdbms_metadata(self, cursor):
        cursor.execute('SELECT version()')
        rs = cursor.fetchone()
        if rs is None:
            result = RDBMSMetadata(VENDOR, PRODUCT, 'unknown')
        else:
            result = RDBMSMetadata(VENDOR, PRODUCT, rs[0])

        return result

    def get_table_metadata(self, table, cursor):
        self._ensure_valid_table(cursor, table)
        dbi = self.get_import()
        cursor.execute('DESC %s' % table)
        rs = cursor.fetchone()
        results = []
        while rs is not None:
            column = rs[0]
            coltype = rs[1]
            null = False if rs[2] == 'NO' else True

            match = self.TYPE_RE.match(coltype)
            if match:
                coltype = match.group(1)
                size = match.group(2)
                if size:
                    size = size[1:-1]
                if coltype in ['varchar', 'char']:
                    max_char_size = size
                    precision = None
                else:
                    max_char_size = None
                    precision = size

            data = TableMetadata(column,
                                 coltype,
                                 max_char_size,
                                 precision,
                                 0,
                                 null)
            results += [data]
            rs = cursor.fetchone()

        return results

    def get_index_metadata(self, table, cursor):
        self._ensure_valid_table(cursor, table)
        dbi = self.get_import()
        cursor.execute('SHOW INDEX FROM %s' % table)
        rs = cursor.fetchone()
        result = []
        columns = {}
        descr = {}
        while rs is not None:
            name = rs[2]
            try:
                columns[name]
            except KeyError:
                columns[name] = []

            columns[name] += [rs[4]]

            # Column 1 is a "non-unique" flag.

            if (not rs[1]) or (name.lower() == 'primary'):
                description = 'Unique'
            else:
                description = 'Non-unique'
            if rs[10] is not None:
                description += ', %s index' % rs[10]
            descr[name] = description
            rs = cursor.fetchone()

        names = columns.keys()
        names.sort()
        for name in names:
            result += [IndexMetadata(name, columns[name], descr[name])]

        return result

    def get_tables(self, cursor):
        cursor.execute('SHOW TABLES')
        table_names = []
        rs = cursor.fetchone()
        while rs is not None:
            table_names += [rs[0]]
            rs = cursor.fetchone()

        return table_names



# $Id$
#
# Nose program for testing grizzled.file classes/functions

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

from grizzled.file import *
from cStringIO import StringIO
import os
import tempfile
import atexit

# ---------------------------------------------------------------------------
# Globals
# ---------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class TestFilePackage(object):

    def testUnlinkQuietly(self):
        fd, path = tempfile.mkstemp()
        os.unlink(path)

        try:
            os.unlink(path)
            assert False, 'Expected an exception'
        except OSError:
            pass

        unlink_quietly(path)

    def testRecursivelyRemove(self):
        path = tempfile.mkdtemp()
        print 'Created directory "%s"' % path

        # Create some files underneath

        touch([os.path.join(path, 'foo'),
               os.path.join(path, 'bar')])

        try:
            os.unlink(path)
            assert False, 'Expected an exception'
        except OSError:
            pass

        recursively_remove(path)

    def testTouch(self):
        path = tempfile.mkdtemp()
        atexit.register(recursively_remove, path)
        f = os.path.join(path, 'foo')
        assert not os.path.exists(f)
        touch(f)
        assert os.path.exists(f)





# NOTE: Documentation is intended to be processed by epydoc and contains
# epydoc markup.

"""
Overview
========

The ``grizzled.misc`` module contains miscellanous functions and classes that
don't seem to fit well in other modules.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import logging

from grizzled.exception import ExceptionWithMessage

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['ReadOnly', 'ReadOnlyObjectError']

# ---------------------------------------------------------------------------
# Constants
# ---------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# Logging
# ---------------------------------------------------------------------------

log = logging.getLogger('grizzled.misc')

# ---------------------------------------------------------------------------
# Public classes
# ---------------------------------------------------------------------------

class ReadOnlyObjectError(ExceptionWithMessage):
    """
    Thrown by ``ReadOnly`` to indicate an attempt to set a field.

    :IVariables:
        field_name : str
            name of the read-only field that could not be set

        message : str
            message to associated with the exception
    """
    def __init__(self, field_name, message):
        ExceptionWithMessage.__init__(self, message)
        self.field_name = field_name

class ReadOnly(object):
    """
    A ``ReadOnly`` object wraps another object and prevents all the contained
    object's fields from being written. Example use:

    .. python::

        from grizzled.misc import ReadOnly
        from grizzled.config import Configuration

        config = Configuration()
        config.read('/path/to/some/file')
        roConfig = ReadOnly(config)

    Any attempt to set fields within ``roConfig`` will cause a
    ``ReadOnlyObjectError`` to be raised.

    The ``__class__`` member of the instantiate ``ReadOnly`` class will be the
    class of the contained object, rather than ``ReadOnly``
    (``Configuration`` in the example). Similarly, the ``isinstance()``
    built-in function will compare against the contained object's class.
    However, the ``type()`` built-in will return the ``ReadOnly`` class
    object.
    """
    def __init__(self, wrapped):
        """
        Create a new ``ReadOnly`` object that wraps the ``wrapped`` object
        and enforces read-only access to it.

        :Parameters:
            wrapped : object
                the object to wrap
        """
        self.wrapped = wrapped

    def __getattribute__(self, thing):
        wrapped = object.__getattribute__(self, 'wrapped')
        result = None
        if thing == 'wrapped':
            result = wrapped
        else:
            result = getattr(wrapped, thing)

        return result

    def __setattr__(self, thing, value):
        if thing == 'wrapped':
            object.__setattr__(self, thing, value)
        else:
            raise ReadOnlyObjectError(thing,
                                      'Attempt to access field "%s" of '
                                      'read-only %s object' %
                                      (thing, self.wrapped.__class__.__name__))

# ---------------------------------------------------------------------------
# Public functions
# ---------------------------------------------------------------------------

# backward compatibality
from grizzled.text import str2bool

def bitcount(num):
    """
    Count the number of bits in a numeric (integer or long) value. This
    method is adapted from the Hamming Weight algorithm, described (among
    other places) at http://en.wikipedia.org/wiki/Hamming_weight

    Works for up to 64 bits.

    :Parameters:
        num : int
            The numeric value

    :rtype: int
    :return: The number of 1 bits in the binary representation of ``num``
    """
    # Put count of each 2 bits into those 2 bits.
    num = num - ((num >> 1) & 0x5555555555555555)

    # Put count of each 4 bits into those 4 bits.
    num = (num & 0x3333333333333333) + ((num >> 2) & 0x3333333333333333)

    # Put count of each 8 bits into those 8 bits.
    num = (num + (num >> 4)) & 0x0f0f0f0f0f0f0f0f

    # Left-most bits.
    return int((num * 0x0101010101010101) >> 56)

    



# $Id: 5d58fcedaae1df2ae53225890e97e09671e1accc $

"""
SQL Server extended database driver.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import os
import sys

from grizzled.db.base import (DBDriver, Error, Warning, TableMetadata,
                              IndexMetadata, RDBMSMetadata)

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class SQLServerDriver(DBDriver):
    """DB Driver for Microsoft SQL Server, using the pymssql DB API module."""

    def get_import(self):
        import pymssql
        return pymssql

    def get_display_name(self):
        return 'SQL Server'

    def do_connect(self,
                   host='localhost',
                   port=None,
                   user='',
                   password='',
                   database='default'):
        dbi = self.get_import()
        self.db_name = database
        if port == None:
            port = '1433'
        return dbi.connect(host='%s:%s' % (host, port),
                           user=user,
                           password=password,
                           database=database)

    def get_tables(self, cursor):
        cursor.execute("select name from %s..sysobjects where xtype = 'U'" %
                       self.db_name)
        table_names = []
        rs = cursor.fetchone()
        while rs is not None:
            table_names += [rs[0]]
            rs = cursor.fetchone()

        return table_names

    def get_rdbms_metadata(self, cursor):
        product = ''
        version = ''
        vendor = 'Microsoft Corporation'
        cursor.execute('xp_msver');
        rs = cursor.fetchone()
        while rs is not None:
            name = rs[1].lower()
            value = rs[3]
            if name == 'productname':
                product = value

            elif name == 'productversion':
                version = value
            
            elif name == 'companyname':
                vendor == value

            rs = cursor.fetchone()

        return RDBMSMetadata(vendor, product, version)

    def get_table_metadata(self, table, cursor):
        self._ensure_valid_table(cursor, table)
        dbi = self.get_import()
        cursor.execute("SELECT column_name, data_type, " \
                       "character_maximum_length, numeric_precision, " \
                       "numeric_scale, is_nullable "\
                       "FROM information_schema.columns WHERE "\
                       "LOWER(table_name) = '%s'" % table)
        rs = cursor.fetchone()
        results = []
        while rs is not None:
            is_nullable = False
            if rs[5] == 'YES':
                is_nullable = True

            data = TableMetadata(rs[0], rs[1], rs[2], rs[3], rs[4], is_nullable)
            results += [data]
            rs = cursor.fetchone()
        return results

    def get_index_metadata(self, table, cursor):
        self._ensure_valid_table(cursor, table)
        dbi = self.get_import()
        cursor.execute("EXEC sp_helpindex '%s'" % table)
        rs = cursor.fetchone()
        results_by_name = {}
        while rs is not None:
            name = rs[0]
            description = rs[1]
            columns = rs[2].split(', ')
            results_by_name[name] = IndexMetadata(name, columns, description)
            rs = cursor.fetchone()

        names = results_by_name.keys()
        names.sort()
        result = []
        for name in names:
            result += [results_by_name[name]]

        return result



#!/usr/bin/env python
# ---------------------------------------------------------------------------

"""
Provides some classes and functions for use with the standard Python
``logging`` module.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import logging
import sys
import textwrap

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['WrappingLogFormatter', 'init_simple_stream_logging']

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class WrappingLogFormatter(logging.Formatter):
    """
    A ``logging`` ``Formatter`` class that writes each message wrapped on line
    boundaries. Here's a typical usage scenario:
    
    .. python::
    
        import logging
        import sys
        from grizzled.log import WrappingLogFormatter

        stderr_handler = logging.StreamHandler(sys.stderr)
        formatter = WrappingLogFormatter(format='%(levelname)s %(message)s")
        stderr_handler.setLevel(logging.WARNING)
        stderr_handler.setFormatter(formatter)
        logging.getLogger('').handlers = [stderr_handler]
    """
    def __init__(self, format=None, date_format=None, max_width=79):
        """
        Initialize a new ``WrappingLogFormatter``.

        :Parameters:
            format : str
                The format to use, or ``None`` for the logging default

            date_format : str
                Date format, or ``None`` for the logging default

            max_width : int
                Maximum line width, or ``None`` to default to 79./
        """
        self.wrapper = textwrap.TextWrapper(width=max_width,
                                            subsequent_indent='    ')
        logging.Formatter.__init__(self, format, date_format)

    def format(self, record):
        s = logging.Formatter.format(self, record)
        result = []
        for line in s.split('\n'):
            result += [self.wrapper.fill(line)]

        return '\n'.join(result)

# ---------------------------------------------------------------------------
# Functions
# ---------------------------------------------------------------------------

def init_simple_stream_logging(level=logging.INFO,
                               streams=None,
                               format=None,
                               date_format=None):
    """
    Useful for simple command-line tools, this method configures the Python
    logging API to:
    
    - log to one or more open streams (defaulting to standard output) and
    - use a ``WrappingLogFormatter``

    :Parameters:
        level : int
            Desired log level

        streams : list
            List of files or file-like objects to which to log, or ``None``
            to log to standard output only
            
        format : str
            Log format to use, or ``None`` to use a reasonable default

        date_format : str
            Date format to use in logging, or ``None`` to use a reasonable
            default
    """
    if not streams:
        streams = [sys.stdout]

    if not format:
        format = '%(asctime)s %(message)s'

    if not date_format:
        date_format = '%H:%M:%S'

    logging.basicConfig(level=level)
    handlers = []

    formatter = WrappingLogFormatter(format=format, date_format=date_format)
    for stream in streams:
        log_handler = logging.StreamHandler(stream)
        log_handler.setLevel(level)
        log_handler.setFormatter(formatter)

        handlers += [log_handler]

    logging.getLogger('').handlers = handlers



# $Id$

"""
Network-related methods and classes.
"""
from __future__ import absolute_import

__docformat__ = 'restructuredtext en'

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import urlparse
import shutil
import tempfile
import urllib2
import logging
import os

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['download']

# ---------------------------------------------------------------------------
# Globals
# ---------------------------------------------------------------------------

log = logging.getLogger('grizzled.net')

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# Functions
# ---------------------------------------------------------------------------

def download(url, directory=None, bufsize=8192):
    """
    Download the specified URL to a directory. This module properly handles
    HTTP authentication for URLs like this one::

        https://user:password@localhost:8080/foo/bar/baz.tgz

    Note, however, that user/password authentication is only supported for
    "http" and "https" URLs.

    :Parameters:
        url : str
            the URL to download
        directory : str
            The directory to receive the downloaded file. If this parameter is
            omitted, ``download()`` will create a temporary directory to
            contain the file.
        bufsize : int
            buffer size to use when reading URL

    :rtype:  tuple
    :return: A (*download_directory*, *downloaded_file*) tuple
    """
    pieces = urlparse.urlparse(url)
    path = pieces.path
    if not directory:
        directory = tempfile.mkdtemp(prefix='download')

    outputPath = os.path.join(directory, os.path.basename(path))

    # Handle user/password explicitly.

    if pieces.scheme.startswith('http') and pieces.username:
        # Initialize basic HTTP authentication for this URL.
        # See http://aspn.activestate.com/ASPN/docs/ActivePython/2.5/howto/urllib2/index.html
        #
        # NOTE: This is necessary because urllib doesn't handle URLs like
        # http://user:password@host:port/...

        # Get the user name and password from the URL.
        user, password = pieces.username, pieces.password

        netloc = pieces.hostname
        if pieces.port:
            pieces.hostname += ':%d' % pieces.port
        newPieces = (pieces.scheme, netloc, pieces.path, pieces.query, 
                     pieces.params, pieces.fragment)
        url = urlparse.urlunparse(newPieces)
        log.debug('Installing authorization handler for URL %s' % url)
        passwordMgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        passwordMgr.add_password(realm=None,
                                 uri=url,
                                 user=user,
                                 passwd=password)
        authHandler = urllib2.HTTPBasicAuthHandler(passwordMgr)
        opener = urllib2.build_opener(authHandler)
        opener.open(url)
        urllib2.install_opener(opener)

    log.debug('Downloading "%s" to "%s"' % (url, outputPath))
    shutil.copyfileobj(urllib2.urlopen(url), open(outputPath, 'wb'), bufsize)

    return (outputPath, directory)



# $Id$

"""
The ``grizzled.text`` package contains text-related classes and modules.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

from StringIO import StringIO

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['hexdump']

# ---------------------------------------------------------------------------
# Constants
# ---------------------------------------------------------------------------

REPEAT_FORMAT = '*** Repeated %d times'

# ---------------------------------------------------------------------------
# Functions
# ---------------------------------------------------------------------------

def hexdump(source, out, width=16, start=0, limit=None, show_repeats=False):
    """
    Produce a "standard" hexdump of the specified string or file-like
    object. The output consists of a series of lines like this::

        000000: 72 22 22 22 4f  53 20 72 6f 75  r'''OS rou
        00000a: 74 69 6e 65 73  20 66 6f 72 20  tines for
        000014: 4d 61 63 2c 20  4e 54 2c 20 6f  Mac, NT, o
        00001e: 72 20 50 6f 73  69 78 20 64 65  r Posix de
        000028: 70 65 6e 64 69  6e 67 20 6f 6e  pending on
        000032: 20 77 68 61 74  20 73 79 73 74   what syst
        00003c: 65 6d 20 77 65  27 72 65 20 6f  em we're o
        000046: 6e 2e 0a 0a 54  68 69 73 20 65  n...This e

    The output width (i.e., the number of decoded characters shown on a
    line) can be controlled with the ``width`` parameter.

    Adjacent repeated lines are collapsed by default. For example::

        000000: 00 00 00 00 00  00 00 00 00 00  ..........
        *** Repeated 203 times
        0007f8: 72 22 22 22 4f  53 20 72 6f 75  r'''OS rou

    This behavior can be disabled via the ``show_repeats`` parameter.

    :Parameters:
        source : str or file
            The object whose contents are to be dumped in hex. The
            object can be a string or a file-like object.

        out : file
            Where to dump the hex output

        width : int
            The number of dumped characters per line

        start : int
            Offset within ``input`` where reading should begin

        limit : int
            Total number of bytes to dump. Defaults to everything from
            ``start`` to the end.

        show_repeats : bool
            ``False`` to collapse repeated output lines, ``True`` to
            dump all lines, even if they're repeats.
    """

    def ascii(b):
        """Determine how to show a byte in ascii."""
        if 32 <= b <= 126:
            return chr(b)
        else:
            return '.'

    pos = 0
    ascii_map = [ ascii(c) for c in range(256) ]

    lastbuf = ''
    lastline = ''
    repeat_count = 0

    if width > 4:
        space_col = width/2
    else:
        space_col = -1

    if type(source) == str:
        source = StringIO(source)

    if start:
        source.seek(start)
        pos = start

    hex_field_width = (width * 3) + 1

    total_read = 0
    while True:
        if limit:
            to_read = min(limit - total_read, width)
        else:
            to_read = width

        buf = source.read(to_read)
        length = len(buf)
        total_read += length
        if length == 0:
            if repeat_count and (not show_repeats):
                if repeat_count > 1:
                    print >> out, REPEAT_FORMAT % (repeat_count - 1)
                elif repeat_count == 1:
                    print >> out, lastline
                print >> out, lastline
            break

        else:
            show_buf = True

            if buf == lastbuf:
                repeat_count += 1
                show_buf = False
            else:
                if repeat_count and (not show_repeats):
                    if repeat_count == 1:
                        print >> out, lastline
                    else:
                        print >> out, REPEAT_FORMAT % (repeat_count - 1)
                    repeat_count = 0

            # Build output line.
            hex = ""
            asc = ""
            for i in range(length):
                c = buf[i]
                if i == space_col:
                    hex = hex + " "
                hex = hex + ("%02x" % ord(c)) + " "
                asc = asc + ascii_map[ord(c)]
            line = "%06x: %-*s %s" % (pos, hex_field_width, hex, asc)

            if show_buf:
                print >> out, line

            pos = pos + length
            lastbuf = buf
            lastline = line

def str2bool(s):
    """
    Convert a string to a boolean value. The supported conversions are:

        +--------------+---------------+
        | String       | Boolean value |
        +==============+===============+
        | "false"      | False         |
        +--------------+---------------+
        | "true"       | True          |
        +--------------+---------------+
        | "f"          | False         |
        +--------------+---------------+
        | "t"          + True          |
        +--------------+---------------+
        | "0"          | False         |
        +--------------+---------------+
        | "1"          + True          |
        +--------------+---------------+
        | "n"          | False         |
        +--------------+---------------+
        | "y"          + True          |
        +--------------+---------------+
        | "no"         | False         |
        +--------------+---------------+
        | "yes"        + True          |
        +--------------+---------------+
        | "off"        | False         |
        +--------------+---------------+
        | "on"         + True          |
        +--------------+---------------+

    Strings are compared in a case-blind fashion.

    **Note**: This function is not currently localizable.

    :Parameters:
        s : str
            The string to convert to boolean

    :rtype: bool
    :return: the corresponding boolean value

    :raise ValueError: unrecognized boolean string
    """
    try:
        return {'false' : False,
                'true'  : True,
                'f'     : False,
                't'     : True,
                '0'     : False,
                '1'     : True,
                'no'    : False,
                'yes'   : True,
                'y'     : False,
                'n'     : True,
                'off'   : False,
                'on'    : True}[s.lower()]
    except KeyError:
        raise ValueError, 'Unrecognized boolean string: "%s"' % s



"""
``grizzled.collections.tuple`` contains some useful tuple-related classes
and functions.
"""
__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

from operator import itemgetter as _itemgetter
from keyword import iskeyword as _iskeyword
import sys as _sys

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['namedtuple']

# ---------------------------------------------------------------------------
# Public Functions
# ---------------------------------------------------------------------------

def namedtuple(typename, fieldnames, verbose=False):
    """
    Returns a new subclass of tuple with named fields. If running under Python
    2.6 or newer, this function is nothing more than an alias for the
    standard Python ``collections.namedtuple()`` function. Otherwise,
    this function is a local implementation, adapted from an
    `ActiveState namedtuple recipe`_.
    
    .. _ActiveState namedtuple recipe: http://code.activestate.com/recipes/500261/

    Usage:
    
    .. python::

        Point = namedtuple('Point', 'x y')
        p0 = Point(10, 20)
        p1 = Point(11, y=22)
        p2 = Point(x=1, y=2)
        print p2[0]          # prints 1
        print p1[1]          # prints 22
        x, y = p0            # x=10, y=20
        print p0.x           # prints 10
        d = p2._asdict()     # convert to dictionary
        print d['x']         # prints 1
        
    :Parameters:
        typename : str
            Name for the returned class
            
        fieldnames : str or sequence
            A single string with each field name separated by whitespace and/or 
            commas (e.g., 'x y' or 'x, y'). Alternatively, ``fieldnames`` can 
            be a sequence of strings such as ['x', 'y'].

        verbose : bool
            If ``True``, the class definition will be printed to standard 
            output before being returned

    :rtype: class
    :return: The named tuple class
    
    :raise ValueError: Bad parameters
    """
    return _namedtuple(typename, fieldnames, verbose=verbose)

# ---------------------------------------------------------------------------
# Private
# ---------------------------------------------------------------------------

def _local_namedtuple(typename, fieldnames, verbose=False):
    # Parse and validate the field names. Validation serves two purposes,
    # generating informative error messages and preventing template injection
    # attacks.

    if isinstance(fieldnames, basestring):
        # names separated by whitespace and/or commas
        fieldnames = fieldnames.replace(',', ' ').split()

    fieldnames = tuple(map(str, fieldnames))
    for name in (typename,) + fieldnames:
        if not min(c.isalnum() or c=='_' for c in name):
            raise ValueError('Type names and field names can only contain '
                             'alphanumeric characters and underscores: %r' % name)

        if _iskeyword(name):
            raise ValueError('Type names and field names cannot be a keyword: '
                             '%r' % name)

        if name[0].isdigit():
            raise ValueError('Type names and field names cannot start with a '
                             'number: %r' % name)

    seen_names = set()
    for name in fieldnames:
        if name.startswith('_'):
            raise ValueError('Field names cannot start with an underscore: '
                             '%r' % name)
        if name in seen_names:
            raise ValueError('Encountered duplicate field name: %r' % name)

        seen_names.add(name)

    # Create and fill-in the class template

    numfields = len(fieldnames)
    argtxt = repr(fieldnames).replace("'", "")[1:-1]   # tuple repr without parens or quotes
    reprtxt = ', '.join('%s=%%r' % name for name in fieldnames)
    dicttxt = ', '.join('%r: t[%d]' % (name, pos) for pos, name in enumerate(fieldnames))
    template = '''class %(typename)s(tuple):
        '%(typename)s(%(argtxt)s)' \n
        __slots__ = () \n
        _fields = %(fieldnames)r \n
        def __new__(cls, %(argtxt)s):
            return tuple.__new__(cls, (%(argtxt)s)) \n
        @classmethod
        def _make(cls, iterable, new=tuple.__new__, len=len):
            'Make a new %(typename)s object from a sequence or iterable'
            result = new(cls, iterable)
            if len(result) != %(numfields)d:
                raise TypeError('Expected %(numfields)d arguments, got %%d' %% len(result))
            return result \n
        def __repr__(self):
            return '%(typename)s(%(reprtxt)s)' %% self \n
        def _asdict(t):
            'Return a new dict which maps field names to their values'
            return {%(dicttxt)s} \n
        def _replace(self, **kwds):
            'Return a new %(typename)s object replacing specified fields with new values'
            result = self._make(map(kwds.pop, %(fieldnames)r, self))
            if kwds:
                raise ValueError('Got unexpected field names: %%r' %% kwds.keys())
            return result \n\n''' % locals()
    for i, name in enumerate(fieldnames):
        template += '        %s = property(itemgetter(%d))\n' % (name, i)

    if verbose:
        print template

    # Execute the template string in a temporary namespace
    namespace = dict(itemgetter=_itemgetter)
    try:
        exec template in namespace
    except SyntaxError, e:
        raise SyntaxError(e.message + ':\n' + template)

    result = namespace[typename]

    # For pickling to work, the __module__ variable needs to be set to the
    # frame where the named tuple is created. Bypass this step in enviroments
    # where sys._getframe is not defined (Jython for example).

    if hasattr(_sys, '_getframe') and _sys.platform != 'cli':
        result.__module__ = _sys._getframe(1).f_globals['__name__']

    return result

# ---------------------------------------------------------------------------
# Initialization
# ---------------------------------------------------------------------------

if _sys.hexversion >= 0x2060000:
    import collections
    _namedtuple = collections.namedtuple
else:
    _namedtuple = _local_namedtuple



# $Id: 8a05a4ac5aa26503d0b05d4e53db7d355ab95f47 $

"""
The *Grizzled Utility Library* is a general-purpose Python library with
a variety of different modules and packages. It's roughly organized into
subpackages that group different kinds of utility functions and classes.

See the `API documentation`_ for complete details.

.. _API documentation: http://www.clapper.org/software/python/grizzled/epydoc
"""

__version__ = '0.9.4'
__author__  = 'Brian M. Clapper'
__email__   = 'bmc@clapper.org'
__url__     = 'http://bmc.github.com/grizzled/'
__license__ = 'BSD license'
__title__   = 'The Grizzled Python Utility Library'

# Public stuff

version = __version__
author  = __author__
email   = __email__
url     = __url__
license = __license__
title   = __title__



# $Id: 85799ff3b36841e5fe97bf09dfc658849b83a71f $

"""
Oracle extended database driver.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import os
import sys

from grizzled.db.base import (DBDriver, Error, Warning,
                              TableMetadata, IndexMetadata, RDBMSMetadata)

# ---------------------------------------------------------------------------
# Constants
# ---------------------------------------------------------------------------

VENDOR = 'Oracle Corporation'
PRODUCT = 'Oracle'

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class OracleDriver(DBDriver):
    """DB Driver for Oracle, using the cx_Oracle DB API module."""

    def get_import(self):
        import cx_Oracle
        return cx_Oracle

    def get_display_name(self):
        return "Oracle"

    def do_connect(self,
                   host='localhost',
                   port=None,
                   user='',
                   password='',
                   database='default'):
        dbi = self.get_import()
        return dbi.connect('%s/%s@%s' % (user, password, database))

    def get_tables(self, cursor):
        cursor.execute('select lower(table_name) from all_tables')
        table_names = []
        rs = cursor.fetchone()
        while rs is not None:
            name = rs[0]
            # Skip tables with "$" in them.
            if name.find('$') < 0:
                table_names.append(name)
            rs = cursor.fetchone()

        return table_names

    def get_rdbms_metadata(self, cursor):
        cursor.execute("SELECT banner FROM v$version WHERE "
                       "banner LIKE 'Oracle%'")
        rs = cursor.fetchone()
        if rs is None:
            result = RDBMSMetadata(VENDOR, PRODUCT, 'unknown')
        else:
            result = RDBMSMetadata(VENDOR, PRODUCT, rs[0])

        return result

    def get_table_metadata(self, table, cursor):
        self._ensure_valid_table(cursor, table)
        cursor.execute("select column_name, data_type, data_length, "
                       "data_precision, data_scale, nullable, "
                       "char_col_decl_length from all_tab_columns "
                       "where lower(table_name) = '%s'" % table.lower())
        results = []
        rs = cursor.fetchone()
        while rs:
            column = rs[0]
            coltype = rs[1]
            data_length = rs[2]
            precision = rs[3]
            scale = rs[4]
            nullable = (rs[5] == 'Y')
            declared_char_length = rs[6]

            if declared_char_length:
                length = declared_char_length
            else:
                length = data_length

            results += [TableMetadata(column,
                                      coltype,
                                      length,
                                      precision,
                                      scale,
                                      nullable)]
            rs = cursor.fetchone()

        return results

    def get_index_metadata(self, table, cursor):
        self._ensure_valid_table(cursor, table)
        # First, issue a query to get the list of indexes and some
        # descriptive information.
        cursor.execute("select index_name, index_type, uniqueness, "
                       "max_extents,temporary from all_indexes where "
                       "lower(table_name) = '%s'" % table.lower())

        names = []
        description = {}
        rs = cursor.fetchone()
        while rs is not None:
            (name, index_type, unique, max_extents, temporary) = rs
            desc = 'Temporary ' if temporary == 'Y' else ''
            unique = unique.lower()
            if unique == 'nonunique':
                unique = 'non-unique'
            index_type = index_type.lower()
            desc += '%s %s index' % (index_type, unique)
            if max_extents:
                desc += ' (max_extents=%d)' % max_extents
            names.append(name)
            description[name] = desc
            rs = cursor.fetchone()

        cursor.execute("SELECT aic.index_name, aic.column_name, "
                       "aic.column_position, aic.descend, aic.table_owner, "
                       "CASE alc.constraint_type WHEN 'U' THEN 'UNIQUE' "
                       "WHEN 'P' THEN 'PRIMARY KEY' ELSE '' END "
                       "AS index_type FROM all_ind_columns aic "
                       "LEFT JOIN all_constraints alc "
                       "ON aic.index_name = alc.constraint_name AND "
                       "aic.table_name = alc.table_name AND "
                       "aic.table_owner = alc.owner "
                       "WHERE lower(aic.table_name) = '%s' "
                       "ORDER BY COLUMN_POSITION" % table.lower())
        rs = cursor.fetchone()
        columns = {}
        while rs is not None:
            index_name = rs[0]
            column_name = rs[1]
            asc = rs[3]
            cols = columns.get(index_name, [])
            cols.append('%s %s' % (column_name, asc))
            columns[index_name] = cols
            rs = cursor.fetchone()

        # Finally, assemble the result.
        results = []
        for name in names:
            cols = columns.get(name, [])
            desc = description.get(name, None)
            results += [IndexMetadata(name, cols, desc)]

        return results




#!/usr/bin/env python
# ---------------------------------------------------------------------------

"""
Provides a front-end to the Python standard ``optparse`` module. The
``CommandLineParser`` class makes two changes to the standard behavior.

  - The output for the '-h' option is slightly different.
  - A bad option causes the parser to generate the entire usage output,
    not just an error message.

It also provides a couple extra utility modules.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

from optparse import OptionParser
import sys

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['CommandLineParser']

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class CommandLineParser(OptionParser):
    """Custom version of command line option parser."""

    def __init__(self, *args, **kw):
        """ Create a new instance. """

        OptionParser.__init__(self, *args, **kw)

        # I like my help option message better than the default...

        self.remove_option('-h')
        self.add_option('-h', '--help', action='help',
                        help='Show this message and exit.')
        
        self.epilogue = None

    def print_help(self, out=sys.stderr):
        """
        Print the help message, followed by the epilogue (if set), to the
        specified output file. You can define an epilogue by setting the
        ``epilogue`` field.
        
        :Parameters:
            out : file
                where to write the usage message
        """
        OptionParser.print_help(self, out)
        if self.epilogue:
            import textwrap
            print >> out, '\n%s' % textwrap.fill(self.epilogue, 80)
            out.flush()

    def die_with_usage(self, msg=None, exit_code=2):
        """
        Display a usage message and exit.

        :Parameters:
            msg : str
                If not set to ``None`` (the default), this message will be
                displayed before the usage message
                
            exit_code : int
                The process exit code. Defaults to 2.
        """
        if msg != None:
            print >> sys.stderr, msg
        self.print_help(sys.stderr)
        sys.exit(exit_code)

    def error(self, msg):
        """
        Overrides parent ``OptionParser`` class's ``error()`` method and
        forces the full usage message on error.
        """
        sys.stderr.write("%s: error: %s\n" % (self.get_prog_name(), msg))
        self.die_with_usage(msg)



# $Id: f39358a756fd212eac2acccfdc7142554cd352ed $

"""
PostgreSQL extended database driver.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import os
import sys
import re

from grizzled.db.base import (DBDriver, Error, Warning, TableMetadata,
                              IndexMetadata, RDBMSMetadata)

# ---------------------------------------------------------------------------
# Constants
# ---------------------------------------------------------------------------

VENDOR  = 'PostgreSQL Global Development Group'
PRODUCT = 'PostgreSQL'

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class PostgreSQLDriver(DBDriver):
    """DB Driver for PostgreSQL, using the psycopg2 DB API module."""

    TYPE_RE = re.compile('([a-z ]+)(\([0-9]+\))?')

    def get_import(self):
        import psycopg2
        return psycopg2

    def get_display_name(self):
        return "PostgreSQL"

    def do_connect(self,
                   host='localhost',
                   port=None,
                   user='',
                   password='',
                   database='default'):
        dbi = self.get_import()
        dsn = 'host=%s dbname=%s user=%s password=%s' %\
            (host, database, user, password)
        return dbi.connect(dsn=dsn)

    def get_rdbms_metadata(self, cursor):
        cursor.execute('SELECT version()')
        rs = cursor.fetchone()
        if rs is None:
            result = RDBMSMetadata(VENDOR, PRODUCT, 'unknown')
        else:
            result = RDBMSMetadata(VENDOR, PRODUCT, rs[0])

        return result

    def get_table_metadata(self, table, cursor):
        self._ensure_valid_table(cursor, table)
        sel = """\
        SELECT a.attname, pg_catalog.format_type(a.atttypid, a.atttypmod),
                    (SELECT substring(d.adsrc for 128)
                     FROM pg_catalog.pg_attrdef d
                     WHERE d.adrelid = a.attrelid AND
                     d.adnum = a.attnum AND a.atthasdef) AS DEFAULT,
                    a.attnotnull,
                    a.attnum,
                    a.attrelid as table_oid
             FROM pg_catalog.pg_attribute a
             WHERE a.attrelid =
             (SELECT c.oid FROM pg_catalog.pg_class c
             LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
             WHERE (pg_table_is_visible(c.oid)) AND c.relname = '%s'
             AND c.relkind in ('r','v'))
             AND a.attnum > 0
             AND NOT a.attisdropped
             ORDER BY a.attnum"""

        cursor.execute(sel % table)
        rs = cursor.fetchone()
        results = []
        while rs is not None:
            column = rs[0]
            coltype = rs[1]
            null = not rs[3]

            match = self.TYPE_RE.match(coltype)
            if match:
                coltype = match.group(1)
                size = match.group(2)
                if size:
                    size = size[1:-1]
                if 'char' in coltype:
                    max_char_size = size
                    precision = None
                else:
                    max_char_size = None
                    precision = size

            data = TableMetadata(column,
                                 coltype,
                                 max_char_size,
                                 precision,
                                 0,
                                 null)
            results += [data]
            rs = cursor.fetchone()

        return results

    def get_index_metadata(self, table, cursor):
        self._ensure_valid_table(cursor, table)
        # First, issue one query to get the list of indexes for the table.
        index_names = self.__get_index_names(table, cursor)

        # Now we need two more queries: One to get the columns in the
        # index and another to get descriptive information.
        results = []
        for name in index_names:
            columns = self.__get_index_columns(name, cursor)
            desc = self.__get_index_description(name, cursor)
            results += [IndexMetadata(name, columns, desc)]

        return results

    def get_tables(self, cursor):

        sel = "SELECT tablename FROM pg_tables " \
              "WHERE tablename NOT LIKE 'pg_%' AND tablename NOT LIKE 'sql\_%'"
        cursor.execute(sel)
        table_names = []
        rs = cursor.fetchone()
        while rs is not None:
            table_names += [rs[0]]
            rs = cursor.fetchone()

        return table_names

    def __get_index_names(self, table, cursor):
        # Adapted from the pgsql command "\d indexname", PostgreSQL 8.
        # (Invoking the pgsql command with -E shows the issued SQL.)

        sel = "SELECT n.nspname, c.relname as \"IndexName\", c2.relname " \
              "FROM pg_catalog.pg_class c " \
              "JOIN pg_catalog.pg_index i ON i.indexrelid = c.oid " \
              "JOIN pg_catalog.pg_class c2 ON i.indrelid = c2.oid " \
              "LEFT JOIN pg_catalog.pg_user u ON u.usesysid = c.relowner " \
              "LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace " \
              "WHERE c.relkind IN ('i','') " \
              "AND n.nspname NOT IN ('pg_catalog', 'pg_toast') " \
              "AND pg_catalog.pg_table_is_visible(c.oid) " \
              "AND c2.relname = '%s'" % table.lower()

        cursor.execute(sel)
        index_names = []
        rs = cursor.fetchone()
        while rs is not None:
            index_names += [rs[1]]
            rs = cursor.fetchone()

        return index_names

    def __get_index_columns(self, index_name, cursor):
        # Adapted from the pgsql command "\d indexname", PostgreSQL 8.
        # (Invoking the pgsql command with -E shows the issued SQL.)

        sel = "SELECT a.attname, " \
              "pg_catalog.format_type(a.atttypid, a.atttypmod), " \
              "a.attnotnull " \
              "FROM pg_catalog.pg_attribute a, pg_catalog.pg_index i " \
              "WHERE a.attrelid in " \
              " (SELECT c.oid FROM pg_catalog.pg_class c " \
              "LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace " \
              " WHERE pg_catalog.pg_table_is_visible(c.oid) " \
              "AND c.relname ~ '^(%s)$') " \
              "AND a.attnum > 0 AND NOT a.attisdropped " \
              "AND a.attrelid = i.indexrelid " \
              "ORDER BY a.attnum" % index_name
        cursor.execute(sel)
        columns = []
        rs = cursor.fetchone()
        while rs is not None:
            columns += [rs[0]]
            rs = cursor.fetchone()

        return columns

    def __get_index_description(self, index_name, cursor):
        sel = "SELECT i.indisunique, i.indisprimary, i.indisclustered, " \
              "a.amname, c2.relname, " \
              "pg_catalog.pg_get_expr(i.indpred, i.indrelid, true) " \
              "FROM pg_catalog.pg_index i, pg_catalog.pg_class c, " \
              "pg_catalog.pg_class c2, pg_catalog.pg_am a " \
              "WHERE i.indexrelid = c.oid AND c.relname ~ '^(%s)$' " \
              "AND c.relam = a.oid AND i.indrelid = c2.oid" % index_name
        cursor.execute(sel)
        desc = ''
        rs = cursor.fetchone()
        if rs is not None:
            if str(rs[1]) == "True":
                desc += "(PRIMARY) "

            if str(rs[0]) == "True":
                desc += "Unique"
            else:
                desc += "Non-unique"

            if str(rs[2]) == "True":
                desc += ", clustered"
            else:
                desc += ", non-clustered"

            if rs[3] is not None:
                desc += " %s" % rs[3]

            desc += ' index'

        if desc == '':
            desc = None

        return desc



# $Id: 17f8764b33a569d116d20fced843f409cf2ba445 $

"""
File Locking
============

This module provides portable advisory file locking primitives that operate on
file descriptors. POSIX-like systems and Windows systems use different
primitives to perform file locking, and these different primitives are modeled
by incompatible (and different) modules in the Python standard library. This
module provides an abstract ``FileLock`` class, and underlying
implementations, to hide the operating system dependencies behind a simple
portable interface.

To create a file lock, simply instantiate the ``FileLock`` class with an open
file descriptor. It handles the rest:

.. python::

    from grizzled.io.filelock import FileLock

    fd = open('/tmp/lockfile', 'r+')
    lock = FileLock(fd)
    lock.acquire()

    ...

    lock.release()

You can also use the ``locked_file()`` function to simplify your code:

.. python::

    from grizzled.io.filelock import locked_file

    fd = open('/tmp/lockfile', 'r+')
    with locked_file(fd):
        ...
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import os
from contextlib import contextmanager

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

__all__ = ['FileLock', 'locked_file']

LOCK_CLASSES = {'posix' : '_PosixFileLock',
                'nt'    : '_WindowsFileLock'}

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class FileLock(object):
    """
    A ``FileLock`` object models a file lock. It wraps a file descriptor
    and contains methods to acquire and release a lock on the file.

    File lock implementations that implement this interface are guaranteed
    to be advisory, but not mandatory, file locks. (They may, in fact, also
    be mandatory file locks, but they are not guaranteed to be.)

    Currently, there are underlying implementations for both POSIX systems
    and Windows.
    """

    def __init__(self, fd):
        """
        Allocate a new file lock that operates on the specified file
        descriptor.

        :Parameters:
            fd : int
                Open file descriptor. The file must be opened for writing or
                updating, not reading.
        """
        try:
            cls = eval(LOCK_CLASSES[os.name])
            self.lock = cls(fd)

        except KeyError:
            raise NotImplementedError, \
                  '''Don't know how to lock files on "%s" systems.''' % os.name

    def acquire(self, no_wait=False):
        """
        Lock the associated file. If someone already has the file locked,
        this method will suspend the calling process, unless ``no_wait`` is
        ``True``.

        :Parameters:
            no_wait : bool
                If ``False``, then ``acquire()`` will suspend the calling
                process if someone has the file locked. If ``True``, then
                ``acquire()`` will raise an ``IOError`` if the file is
                locked by someone else.

        :raise IOError: If the file cannot be locked for any reason.
        """
        self.lock.acquire(no_wait)

    def release(self):
        """
        Unlock (i.e., release the lock on) the associated file.
        """
        self.lock.release()

class _PosixFileLock(object):
    """File lock implementation for POSIX-compliant systems."""

    def __init__(self, fd):
        self.fd = fd

    def acquire(self, no_wait=False):
        import fcntl
        flags = fcntl.LOCK_EX
        if no_wait:
            flags |= fcntl.LOCK_NB

        fcntl.lockf(self.fd, flags)

    def release(self):
        import fcntl
        fcntl.lockf(self.fd, fcntl.LOCK_UN)

class _WindowsFileLock(object):
    """File lock implementation for Windows systems."""

    def __init__(self, fd):
        self.fd = fd

    def lock(self, no_wait=False):
        import msvcrt
        if no_wait:
            op = msvcrt.LK_NBLCK
        else:
            op = msvcrt.LK_LOCK

        self.fd.seek(0)
        msvcrt.locking(self.fd, op, 1)

    def unlock(self):
        import msvcrt
        self.fd.seek(0)
        msvcrt.locking(self.fd, LK_UNLCK, 1)

# ---------------------------------------------------------------------------
# Functions
# ---------------------------------------------------------------------------

@contextmanager
def locked_file(fd, no_wait=False):
    """
    This function is intended to be used as a ``with`` statement context
    manager. It wraps a ``FileLock`` object so that the locking and unlocking
    of the file descriptor are automatic. With the ``locked_file()`` function,
    you can replace this code:
    
    .. python::

        lock = FileLock(fd)
        lock.acquire()
        try:
            do_something()
        finally:
            lock.release()

    with this code:
    
    .. python::

        with locked_file(fd):
            do_something()

    :Parameters:
        fd : int
            Open file descriptor. The file must be opened for writing
            or updating, not reading.
        no_wait : bool
            If ``False``, then ``locked_file()`` will suspend the calling
            process if someone has the file locked. If ``True``, then
            ``locked_file()`` will raise an ``IOError`` if the file is
            locked by someone else.
    """
    locked = False
    try:
        lock = FileLock(fd)
        lock.acquire(no_wait)
        locked = True
        yield lock
    finally:
        if locked:
            lock.release()



# $Id$
#
# Nose program for testing grizzled.sys classes/functions

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

from __future__ import absolute_import

import sys
from grizzled.system import *

# ---------------------------------------------------------------------------
# Globals
# ---------------------------------------------------------------------------

VERSIONS = [('2.5.1', 0x020501f0),
            ('1.5',   0x010500f0),
            ('2.6',   0x020600f0),
            ('2.4.3', 0x020403f0)]

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class TestSys(object):

    def test_version_conversions(self):
        for s, i in VERSIONS:
            yield self.do_one_version_conversion, s, i
            
    def do_one_version_conversion(self, string_version, binary_version):
        h = python_version(string_version)
        s = python_version_string(binary_version)
        assert h == binary_version
        assert s == string_version
        
    def test_current_version(self):
        ensure_version(sys.hexversion)
        ensure_version(python_version_string(sys.hexversion))
        major, minor, patch, final, rem = sys.version_info
        binary_version = python_version('%d.%d.%d' % (major, minor, patch))

    def test_class_for_name(self):
        cls = class_for_name('grizzled.config.Configuration')
        got_name = '%s.%s' % (cls.__module__, cls.__name__)
        assert got_name == 'grizzled.config.Configuration'

        try:
            class_for_name('grizzled.foo.bar.baz')
            assert False
        except NameError:
            pass
        except ImportError:
            pass



# $Id: 51fcf8b2cf945a965c835f871882ce88d6a4d3ce $

"""
Gadfly extended database driver.
"""

__docformat__ = "restructuredtext en"

# ---------------------------------------------------------------------------
# Imports
# ---------------------------------------------------------------------------

import os
import sys

from grizzled.db.base import (Cursor, DB, DBDriver, Error, Warning,
                              TableMetadata, IndexMetadata, RDBMSMetadata)

# ---------------------------------------------------------------------------
# Exports
# ---------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# Classes
# ---------------------------------------------------------------------------

class GadflyCursor(Cursor):
    def __init__(self, real_cursor, driver):
        self.real_cursor = real_cursor
        self.driver = driver

    @property
    def rowcount(self):
        total = len(self.real_cursor.fetchall())
        self.real_cursor.reset_results()
        return total

    @property
    def description(self):
        return self.real_cursor.description

    def close(self):
        try:
            self.real_cursor.close()
        except:
            raise Error(sys.exc_info()[1])

    def execute(self, statement, parameters=None):
        try:
            if parameters:
                result = self.real_cursor.execute(statement, parameters)
            else:
                result = self.real_cursor.execute(statement)
            return result
        except:
            raise Error(sys.exc_info()[1])

    def executemany(self, statement, *parameters):
        try:
            return self.real_cursor.executemany(statement, *parameters)
        except:
            raise Error(sys.exc_info()[1])

    def fetchall(self):
        try:
            return self.real_cursor.fetchall()
        except:
            raise Error(sys.exc_info()[1])

    def fetchone(self):
        try:
            return self.real_cursor.fetchone()
        except:
            s = sys.exc_info()[1]
            if (type(s) == str) and (s.startswith('no more')):
                return None
            raise Error(s)

    def fetchmany(self, n):
        try:
            return self.real_cursor.fetchmany(n)
        except:
            s = sys.exc_info()[1]
            if (type(s) == str) and (s.startswith('no more')):
                return None
            raise Error(s)

class GadflyDB(DB):
    def __init__(self, db, driver):
        DB.__init__(self, db, driver)
        self.__db = db
        self.__driver = driver

    def cursor(self):
        return Cursor(GadflyCursor(self.__db.cursor(), self.__driver),
                      self.__driver)

class GadflyDriver(DBDriver):
    """DB Driver for Gadfly, a pure Python RDBMS"""

    def __init__(self):
        gadfly = self.get_import()
        gadfly.error = Exception()

    def get_import(self):
        import gadfly
        return gadfly

    def get_display_name(self):
        return "Gadfly"

    def connect(self,
                host=None,
                port=None,
                user='',
                password='',
                database='default'):
        gadfly = self.get_import()
        directory = os.path.dirname(database)
        database = os.path.basename(database)
        if database.endswith('.gfd'):
            database = database[:-4]

        try:
            g = gadfly.gadfly()
            g.startup(database, directory)
            return GadflyDB(g, self)
        except IOError:
            raise Error(sys.exc_info()[1])

    def get_tables(self, cursor):
        cursor.execute('SELECT table_name FROM __table_names__ '
                       'WHERE is_view = 0')
        table_names = []
        for row in cursor.fetchall():
            table_names += [row[0]]

        return table_names

    def get_rdbms_metadata(self, cursor):
        import gadfly
        version = '.'.join([str(i) for i in gadfly.version_info])
        return RDBMSMetadata('gadfly', 'gadfly', version)

    def get_table_metadata(self, table, cursor):
        self._ensure_valid_table(cursor, table)

        cursor.execute("SELECT column_name FROM __columns__ "
                       "WHERE table_name = '%s'" % table.upper())
        result = []
        column_names = []
        for row in cursor.fetchall():
            result += [TableMetadata(row[0], 'object', None, None, None, True)]
        return result

    def get_index_metadata(self, table, cursor):
        self._ensure_valid_table(cursor, table)

        cursor.execute("SELECT is_unique, index_name FROM __indices__ "
                       "WHERE table_name = '%s'" % table.upper())
        indexes = []
        result = []
        for row in cursor.fetchall():
            indexes.append(row)

        for unique, index_name in indexes:
            cursor.execute("SELECT column_name FROM __indexcols__ "
                           "WHERE index_name = '%s'" % index_name)
            cols = []
            for row in cursor.fetchall():
                cols.append(row[0])

            if unique:
                description = 'UNIQUE'
            else:
                description = 'NON-UNIQUE'

            result.append(IndexMetadata(index_name, cols, description))

        return result

    def _is_valid_table(self, cursor, table_name):
        tables = self.get_tables(cursor)
        return table_name.upper() in tables

